prefix = "0.0057"
labels = ['Γ',"M","K",'Γ']


import numpy as np
import matplotlib.pyplot as plt

def Symmetries(fstring):
    f = open(fstring, 'r')
    x = np.zeros(0)
    for i in f:
        if "high-symmetry" in i:
            x = np.append(x, float(i.split()[-1]))
    f.close()
    return x

freq = np.loadtxt("./{}.freq.gp".format(prefix))
load_sym = Symmetries("./plotband.out")
sym = load_sym/max(load_sym)
ph_path = freq.T[0]/max(freq.T[0])
cm2mev = 0.124

fig = plt.figure(figsize=(7,6))
# colors = ["blue","orange","green","red","purple","teal"]
for i in sym[:-1]:
    plt.axvline(i,linestyle="--",color="black")
for i in range(1,len(freq.T)):
    plt.plot(ph_path,freq.T[i]*cm2mev,linewidth=2,color="blue")
plt.xticks(sym,labels,fontsize=15)
# plt.axhline(0,color="black")
plt.title(f"{prefix}")
plt.ylim(0,)
plt.xlim(0,ph_path[-1])
plt.ylabel("ω (meV)",fontsize=15)
plt.savefig('phonon_band.png')
plt.show()


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import os
import re
import subprocess
import json
import qcelemental as qcel
import untangle
import copy
import math
import glob
from . import compute
from .config import default_config

def shift_cell(atom,cell,direction,vector):
    vacuum_atom = np.dot(atom,cell)+vector
    directions = ['x','y','z']
    shift_matrix = np.zeros(shape=(3,3))
    for i,j in enumerate(directions):
        if direction==j:
            shift_matrix[i]=np.array(vector)*2
    cell_vacuum = cell+shift_matrix
    frac_vacuum = np.dot(vacuum_atom,np.linalg.inv(cell_vacuum))

    return frac_vacuum,cell_vacuum



def shift_frac(atoms,vector):
    return np.mod(atoms+vector,1)

def make_monolayer(atoms,direction='z'):
    direction = direction.lower()
    if direction not in ['x','y','z']:
        raise Exception("Direction should be x y or z")
    df = pd.DataFrame()
    atoms=np.array(atoms)
    df['Atoms'] = atoms.T[0]
    df['x'] = atoms.T[1].astype(float)
    df['y'] = atoms.T[2].astype(float)
    df['z'] = atoms.T[3].astype(float)
    df = df.query(f'{direction}<0.5')
    df_shifted = pd.DataFrame()
    df_shifted["Atoms"] = df['Atoms'].values
    for i in ['x','y','z']:
        if direction==i:
            df_shifted[i] = df[i].values+0.25
        else:
            df_shifted[i] = df[i].values
    return df_shifted.values


def plot_sigma_energy(path):
    out_files = os.listdir(path)
    total_energy = []
    e_fermi = []
    for file in out_files:
        temp_file = open(f'{path}/{file}', 'r').readlines()
        for i in range(len(temp_file)):
            if len(temp_file[i].split())>2:
                if temp_file[i].split()[0]=='!':
                    temp_en=temp_file[i].split()[4]
                # if temp_file[i].split()[0]=='the':
                #     temp_fermi=temp_file[i].split()[-2]
        total_energy.append([float(temp_en),float(file[:-13])])
        # e_fermi.append([float(temp_fermi),float(file[:-13])])
        
    tot_en = np.array(total_energy)
    tot_en = tot_en[tot_en[:, 1].argsort()]
    
    
    # e_f = np.array(e_fermi)
    # e_f = e_f[e_f[:, 1].argsort()]
    
    fig = plt.figure(figsize=(8,6))
    plt.plot(tot_en.T[1],tot_en.T[0])
    plt.ylabel('Total Energy [Ry]',fontsize=20)
    plt.xlabel('Smearing Value [Ry]',fontsize=20)
    plt.xticks(fontsize=20)
    plt.yticks(fontsize=20)
    plt.savefig('total_sigma.png')

def get_total_energy(self):
    # path = f'./Projects/{self.project_id}/{self.job_id}/{self.job_id}.save/data-file-schema.xml'
    # obj = untangle.parse(path)
    path = f'./Projects/{self.project_id}/{self.job_id}/scf.out'
    temp_file = open(f'{path}', 'r').readlines()
    for i in range(len(temp_file)):
        if len(temp_file[i].split())>2:
            if temp_file[i].split()[0]=='!':
                en=temp_file[i].split()[4]
    # en = float(obj.qes_espresso.output.total_energy.etot.cdata)*2
    # p = subprocess.Popen(f"grep '!' {path}", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    # line = p.stdout.readlines()[0].decode()
    # en = float(re.findall(r"[-+]?(?:\d*\.*\d+)", line)[0])
    return(en)

def configure(path):
    if not path:
        config = default_config
    else:
        with open(path) as f:
            data = f.read()
            config = json.loads(data)
    return config


def fm_maker(self,magnetic_atom,mag_start=1,angle1=False,angle2=False):
    model = copy.deepcopy(self)
    model.job_id = 'fm'
    model.magnetism=True
    model.magnetic_atom=magnetic_atom
    model.magnetic_order='fm'
    for j,i in enumerate(model.config['pw']['atomic_species']): #for each atom
        if  i['atom']==magnetic_atom: # choose magnetic atom
            model.config['pw']['system'][f'starting_magnetization({j+1})']=mag_start
            if (angle1):
                model.config['pw']['system'][f'angle1({j+1})']=angle1
                self.angle1=angle1
            if (angle2):
                model.config['pw']['system'][f'angle2({j+1})']=angle2
                self.angle2=angle2
            if angle1 or angle2:
                model.config['pw']['system']['noncolin']='true'
            else:
                model.config['pw']['system']['nspin']=2
    ntyp = len(model.config['pw']['atomic_species'])
    model.config['pw']['system']['ntyp']=ntyp
    return model #send it back


def afm_maker(self,magnetic_atom,mag_start=[1,-1],angle1=False,angle2=False):
    #1 - Give initial model
    #2 - Define magnetic atom
    #3 - Return magnetic states
    #4 - FM and AFM
    model = copy.deepcopy(self)
    model.magnetic_atom=magnetic_atom
    model.magnetic_order='afm'
    model.magnetism=True
    """
    FM is easier to do. Only starting magnetization parameter should be added for the relevant atom
    Number of AFM states can be changing depending on the number of magnetic atoms. 
    The number of magnetic atoms should be divided into two for spin up and down. 
    Total magnetism is going to be zero but the order of spins are going to be change.
    """
    models=[] #initialize models
    atom_index=[] #initialize atom index
    for j,i in enumerate(model.config['pw']['atomic_positions']): #iterate over atomic positions
        if i[0]==magnetic_atom: #select magnetic atoms
            atom_index.append(j) #add magnetic atoms index to the array
    number = len(atom_index) # check for magnetic atom number
    if number ==2: #if there are 2
        spin_matrix = [[1,0]] #use this one
    else: #otherwise
        num_config = int(math.factorial(number)/(math.factorial(number-int(number/2))*math.factorial(int(number/2)))/2) #choose N/2 of N divide by 2
        one = np.ones(num_config).T.reshape(num_config,1) #keep the first atom up
        id = np.identity(num_config) #create other atom combinations
        spin_matrix = np.concatenate((one,id), axis=1) #combine them
    for j,i in enumerate(model.config['pw']['atomic_species']): #for each atom
        if  i['atom']==magnetic_atom: # choose magnetic atom
            i['atom']=i['atom']+str(int(1)) #change the name atom atom1
            model.config['pw']['system'][f'starting_magnetization({j+1})']=mag_start[0]
            if (angle1):
                model.config['pw']['system'][f'angle1({j+1})']=angle1
            if (angle2):
                model.config['pw']['system'][f'angle2({j+1})']=angle2
            if angle1 or angle2:
                model.config['pw']['system']['noncolin']='true'
            else:
                model.config['pw']['system']['nspin']=2            
            model.config['pw']['atomic_species'].append(copy.deepcopy(i)) #create the same atom
    model.config['pw']['atomic_species'][-1]['atom']=magnetic_atom+str(int(0)) #change name to atom0
    ntyp = len(model.config['pw']['atomic_species'])
    model.config['pw']['system']['ntyp']=ntyp
    
    for i in model.config['pw']['hubbard']['terms']:
        if i['atom'].lower()==magnetic_atom.lower():
            model.config['pw']['hubbard']['terms'].append(copy.deepcopy(i))
            i['atom']=magnetic_atom+str(int(1))
            model.config['pw']['hubbard']['terms'][-1]['atom']=magnetic_atom+str(int(0))
        
    
    if (angle1):
        model.config['pw']['system'][f'angle1({ntyp})']=angle1
    if (angle2):
        model.config['pw']['system'][f'angle2({ntyp})']=angle2
    model.config['pw']['system'][f'starting_magnetization({ntyp})']=mag_start[1]
    for i,spin_config in enumerate(spin_matrix): #for each configuration of spin matirx
        temp_model = copy.deepcopy(model) #create a model
        temp_model.job_id = f'afm{i+1}'
        for j,i in enumerate(atom_index): #for each magnetic atom
            temp_model.config['pw']['atomic_positions'][i][0]=temp_model.config['pw']['atomic_positions'][i][0]+str(int(spin_config[j])) #change name with the spin matrix
        models.append(temp_model) #add to models array
    return models #send it back

def default_pseudo(atom):
    atom_type = list(set([a[0] for a in atom]))
    atom_array = []
    for i in atom_type:
        for j in i:
            try:
                int(j)
                k=i[:-1]
            except:
                k=i
        temp_atom = {"atom":i,'mass':str(qcel.periodictable.to_mass(k)),'pseudopotential':f"{k}.UPF"}
        atom_array.append(temp_atom)
        # print(temp_atom)
    # print(atom_array)  
    return atom_array  

def atom_type(atom):
    num_type = len(list(set([a[0] for a in atom])))
    return num_type


def test_parameter(self,parameter_name,start,end,step,conv_thr=False,num_core=1,debug=False,out=False,dual=4):
    parameter = np.arange(start,end,step)
    result = np.zeros(shape=(3,len(parameter)))
    end=0
    for j,i in enumerate(parameter):
        if parameter_name=="ecutwfc":
            self.ecutwfc(i)
            self.ecutrho(dual*i)
            self.job_id=f"ecutwfc_{i}"
        if parameter_name=="kpoints":
            self.k_points(int(i))
            self.job_id=f"kpoints_{i}"
        if parameter_name=="num_core":
            self.job_id=f"num_core_{i}"
            self.num_core
        if debug==False:
            self.calculate('scf')
        temp_en = get_total_energy(self)
        temp_time = get_time(self)
        result[0][j]=i #parameters
        result[1][j]=temp_en #energy
        result[2][j]=temp_time #time
        end+=1
        if j!=0:
            if out==True:
                if parameter_name=="num_core":
                    print(f"{parameter_name}: {result[0][j]}     DeltaT :{(result[2][j]-result[2][j-1])} seconds    Time: {result[2][j]} seconds")
                else:
                    print(f"{parameter_name}: {result[0][j]}     DeltaE :{(result[1][j]-result[1][j-1])} seconds    Time: {result[2][j]} seconds")
            if conv_thr!=False:
                if abs(result[1][j-1] - result[1][j]) < conv_thr:
                    end=j
                    break
        else:
            print(f"{parameter_name}: {result[0][j]}      Time: {result[2][j]} seconds")

    # print(result.T[:end+1].T)
    return result.T[:end+1].T


def get_time(self):
    path = f'./Projects/{self.project_id}/{self.job_id}/{self.job_id}.save/data-file-schema.xml'
    obj = untangle.parse(path)
    time = float(obj.qes_espresso.timing_info.total.wall.cdata)
    return time
def pdos_loader(fname):
    import numpy as np

    data = np.loadtxt(fname)
    energy = data[:, 0]
    pdos = data[:, 1]  # ldos col, total contribution for a given orbital

    return energy, pdos


def sumpdos(self):
    files = glob.glob(f'./Projects/{self.project_id}/{self.job_id}/projwfc.dat*')
    for i in files:
        parts = i.split("_")
        if len(parts)==2:
            label='Total'
        else:
            atom = parts[1].split('(')[-1].split(')')[0]
            orbital = parts[2].split('(')[-1].split(')')[0]
            # print(atom,orbital)
            compute.sumpdos(self,atom,orbital)


def sumkdos(self):
    files = glob.glob(f'./Projects/{self.project_id}/{self.job_id}/dos.k*')
    for i in files:
        parts = i.split("_")
        if len(parts)==2:
            label='Total'
        else:
            atom = parts[1].split('(')[-1].split(')')[0]
            orbital = parts[2].split('(')[-1].split(')')[0]
            compute.sumkdos(self,atom,orbital)


def minimum_energy(models):
    energies = []
    labels =[]
    index =[]
    for i,state in enumerate(models):
        energy = get_total_energy(state)
        energies.append(float(energy))
        labels.append(state.job_id.upper())
        index.append(i)
    min_state= np.argmin(energies)
    print(labels[min_state])
    return models[min_state]



default_config = {'pw': {'control': {'pseudo_dir': './pseudos/NC'},
  'system': {'degauss': '0.1', 'ecutwfc': '20', 'occupations': 'smearing'},
  'electrons': {'conv_thr': '1e-6'},
  'ions': {},
  'cell': {},
  'k_points': '1 1 1 0 0 0',
  'hubbard': {'projection': 'atomic', 'terms': []}},
 'dos': {'dos': {'emin': '-20.0', 'emax': '20.0'}},
 'bands': {'bands': {}},
 'projwfc': {'projwfc': {}},
 'ph': {'inputph': {'ldisp': 'true', 'nq1': '1', 'nq2': '1', 'nq3': '1'}},
 'q2r': {'input': {'zasr': 'simple'}},
 'matdyn': {'input': {'asr': 'simple',
   'q_in_band_form': 'true',
   'q_in_cryst_coord': 'true'}}}


import argparse
import numpy as np
import matplotlib.pyplot as plt


ap = argparse.ArgumentParser()
ap.add_argument("-n", "--name", required=True,
                help="Project name")
ap.add_argument("-d", "--degauss", required=True, help="Degauss value")
args = vars(ap.parse_args())

project_name = args['name']
degauss = args['degauss']


def read_efermi(path):
    lines = open(path, 'r').readlines()
    e_fermi = 0
    for i in lines:
        if "the Fermi energy is" in i:
            e_fermi = float(i.split()[-2])
            return e_fermi
        
def Symmetries(fstring):
    f = open(fstring, 'r')
    x = np.zeros(0)
    for i in f:
        if "high-symmetry" in i:
            x = np.append(x, float(i.split()[-1]))
    f.close()
    return x

sym = Symmetries(f'./{project_name}/{degauss}/bands-pp.out')


temp_file = open(f'./{project_name}/{degauss}/scf.out', 'r').readlines()
for i in range(len(temp_file)):
    if len(temp_file[i].split())>2:
        if temp_file[i].split()[0]=='the':
            fermi=temp_file[i].split()[-2]

fig = plt.figure(figsize=(8,6))
data =np.loadtxt(f'./{project_name}/{degauss}/bands.dat.gnu')

k = np.unique(data[:, 0])
bands = np.reshape(data[:, 1], (-1, len(k)))
for band in range(len(bands)):
    plt.plot(k, bands[band, :]-float(fermi),c='black')
plt.xticks(sym,['G','M','K','G'])
plt.axvline(sym[1],c='black')
plt.axvline(sym[2],c='black')
plt.axhline(0,c='red')
plt.ylim(-10,10)
plt.xlim(sym[0],sym[-1])
plt.savefig(f'./{project_name}/plots/band_{degauss}.png')
# plt.show()
                


"""
The seekpath module contains routines to get automatically the
path in a 3D Brillouin zone to plot band structures.

Author: Giovanni Pizzi, PSI (2016-2023)

Licence: MIT License, see LICENSE.txt file
"""

__version__ = "2.1.0"
__author__ = "Giovanni Pizzi, PSI"
__copyright__ = (
    "Copyright (c), 2016-2023, Giovanni Pizzi, PAUL SCHERRER INSTITUT "
    "(Laboratory for Materials Simulations), EPFL "
    "(Theory and Simulation of Materials (THEOS) and National Centre "
    "for Computational Design and Discovery of Novel Materials "
    "(NCCR MARVEL)), Switzerland."
)
__credits__ = ["Yoyo Hinuma", "Jae-Mo Lihm"]
__license__ = "MIT license"
__paper__ = (
    "Y. Hinuma, G. Pizzi, Y. Kumagai, F. Oba, I. Tanaka, "
    "Band structure diagram paths based on crystallography, "
    "Comp. Mat. Sci. 128, 140 (2017). DOI: 10.1016/j.commatsci.2016.10.015"
)


class SupercellWarning(UserWarning):
    """
    A warning issued when the cell is an undistorted supercell of a smaller
    unit cell, and the kpoint path for a non-standardized cell (i.e., for the
    original cell) is requested.
    """

    pass


from .getpaths import (
    get_path,
    get_explicit_k_path,
    get_path_orig_cell,
    get_explicit_k_path_orig_cell,
)

from .hpkot import EdgeCaseWarning, SymmetryDetectionError
from .brillouinzone import brillouinzone
from . import hpkot

__all__ = (
    "get_path",
    "get_explicit_k_path",
    "get_path_orig_cell",
    "get_explicit_k_path_orig_cell",
    "EdgeCaseWarning",
    "SymmetryDetectionError",
    "SupercellWarning",
    "brillouinzone"
)



"""Module to compute the Brillouin zone of a crystal."""
from collections import defaultdict

import numpy as np
from scipy.spatial import Voronoi, ConvexHull, Delaunay


def get_missing_point(tr, p1, p2):
    """
    tr is a list of 3 indices (the indices of the vertices of a given triangle).
    p1 and p2 must be in the tr list, the third index is returned.

    ValueError is raised if more than one entry is not among p1 and p2,
    or if all points are among p1 and p2 (e.g. a point repeated twice)
    """
    missing = None
    for vertex in tr:
        if vertex not in (p1, p2):
            if missing is not None:
                raise ValueError("Two missing points found...")
            missing = vertex

    if missing is None:
        raise ValueError("No missing points!")

    return missing


def are_coplanar(v1, v2, v3):
    """
    v1, v2, v3: 3D vectors.
    Return True if they are coplanar, False otherwise
    """
    return float(abs(np.dot(np.cross(v1, v2), v3))) < 1.0e-6


def get_BZ(b1, b2, b3):
    """
    Get the faces of the BZ given the three vectors b1,b2,b3.
    Return both triangular faces (oriented) and flat faces (if your
        plotting library prefers these - these are not oriented for the
        time being)
    """
    ret_data = {}

    supercell_size = 3  # Is this enough?

    # G-vectors
    points3d = []
    central_idx = None
    for i in range(-supercell_size, supercell_size + 1):
        for j in range(-supercell_size, supercell_size + 1):
            for k in range(-supercell_size, supercell_size + 1):
                if i == 0 and j == 0 and k == 0:
                    central_idx = len(points3d)
                points3d.append(i * np.array(b1) + j * np.array(b2) + k * np.array(b3))

    # Get Voronois
    vor3d = Voronoi(np.array(points3d))
    # Get the vertices of the central" Voronoi( around the origin G=0)
    central_voronoi_3d = np.array(
        [vor3d.vertices[idx] for idx in vor3d.regions[vor3d.point_region[central_idx]]]
    )

    # Get the convex hull of these points (all triangular faces)
    hull = ConvexHull(central_voronoi_3d)

    ## REORIENT TRIANGLES
    ## NOTE! TODO: I should do the same for faces
    ret_data["triangles_vertices"] = hull.points.tolist()
    ## Naive one
    # ret_data['triangles'] = hull.simplices.tolist()
    ## Instead, I orient them all
    ret_data["triangles"] = []
    for simplex in hull.simplices:
        points = np.array([hull.points[i] for i in simplex])
        center = points.sum(axis=0) / float(len(points))

        # Get normal vector (with direction!)
        normal = np.cross(center - points[1], center - points[0])
        # Normalize, then rescale to a small value
        normal = normal / np.linalg.norm(normal)
        max_length = np.sqrt((points**2).sum(axis=1).max())
        normal /= max_length
        normal *= 1.0e-4
        point_up = center + normal
        point_down = center - normal
        delaunay = Delaunay(hull.points)
        is_up_inside = delaunay.find_simplex(point_up) >= 0
        is_down_inside = delaunay.find_simplex(point_down) >= 0

        if is_up_inside and not is_down_inside:
            correct_orientation = True
        elif not is_up_inside and is_down_inside:
            correct_orientation = False
        else:
            inside_outside_string = "inside" if is_up_inside else "outside"
            print(
                "WARNING! Both vectors are {}..."
                " not changing orientation".format(inside_outside_string)
            )
            correct_orientation = True

        if correct_orientation:
            ret_data["triangles"].append(simplex.tolist())
        else:
            ret_data["triangles"].append(simplex[::-1].tolist())

    # print hull.area, hull.volume

    # Get edge-sharing faces
    # edges has as key a tuple (sorted) with the indices of the two vertices of
    # the shared edge; the value are the indices of the triangles
    edges = defaultdict(list)
    for simplex_idx, simplex in enumerate(hull.simplices):
        edges[tuple(sorted([simplex[0], simplex[1]]))].append(simplex_idx)
        edges[tuple(sorted([simplex[1], simplex[2]]))].append(simplex_idx)
        edges[tuple(sorted([simplex[2], simplex[0]]))].append(simplex_idx)
    # convert to dictionary of lists (from defaultdict of sets)
    edges = dict(edges)

    ### Create now the list of faces, merging the triangles that share an
    ### edge and are coplanar. Note: this works only if up to two triangles
    ### must be merged; if three or more, this will not produce the expected
    ### result
    # print edges

    # Store merge operations to perform
    merge_with = defaultdict(set)

    # List of found simplices that have been merged; will be used at the
    # end to add the triangles that have not been merged, if any
    merged_simplices = []
    for (p1, p2), triangles in edges.items():
        # I do it many times, but it's the easiest way (and anyway it's
        # a set, so it should be fast: I add a point to be merged with
        # itself
        merge_with[triangles[0]].add(triangles[0])
        merge_with[triangles[1]].add(triangles[1])

        if len(triangles) != 2:
            # An edge shared by less (or more) than 2 triangles?
            print("Warning!", p1, p2, triangles)
            continue
        else:
            # Check if two triangles are coplanar: get the other two
            # vertices that are not on the shared edge
            otherpoint0 = get_missing_point(hull.simplices[triangles[0]], p1, p2)
            otherpoint1 = get_missing_point(hull.simplices[triangles[1]], p1, p2)
            # The actual vector coordinates
            otherpoint0_p = hull.points[otherpoint0]
            otherpoint1_p = hull.points[otherpoint1]
            p1_p = hull.points[p1]
            p2_p = hull.points[p2]

            # Check if they are coplanar
            if are_coplanar(p2_p - p1_p, otherpoint0_p - p1_p, otherpoint1_p - p1_p):
                merge_with[triangles[0]].add(triangles[1])
                merge_with[triangles[1]].add(triangles[0])

    # PROBLEM TO SOLVE: we have to put together all groups
    ## E.g. we have now:
    # 0: [0, 3]
    # 1: [1, 2, 3]
    # 2: [1, 2]
    # 3: [0, 1, 3]
    ## We should get instead for all [0,1,2,3]
    # So we have to do a pass to merge them all
    # The algorithm below is probably wrong (actually, it is only
    # probably inefficient)

    # for k, v in merge_with.items():
    #    print "{}: {}".format(k, list(v))

    # Iterate untile convergence - not sure this is correct
    has_changed = True
    while has_changed:
        has_changed = False
        # Add the missing ones
        for tr in range(len(hull.simplices)):
            for other1 in merge_with[tr]:
                for other2 in merge_with[tr]:
                    if other1 not in merge_with[other2]:
                        has_changed = True
                        merge_with[other2].add(other1)
                    if other2 not in merge_with[other1]:
                        has_changed = True
                        merge_with[other1].add(other2)

    # convert to dict, and most importantly convert to list and sort
    merge_with = {k: sorted(v) for k, v in merge_with.items()}

    # Assign to the smallest integer idx
    merge_group = {k: v[0] for k, v in merge_with.items()}

    groups = defaultdict(list)
    # I create a reverse index
    for k, v in merge_group.items():
        groups[v].append(k)

    # List of faces (elements are lists of vertex ids)
    faces = []

    for group in groups.values():
        if len(group) == 1:
            faces.append(
                [hull.points[point_idx] for point_idx in hull.simplices[group[0]]]
            )
        else:
            # Get all points
            all_points_idx = sorted(
                set(np.concatenate([hull.simplices[g] for g in group]))
            )

            all_points_coords = [hull.points[point_idx] for point_idx in all_points_idx]
            # Find projection in 2D: I first choose a first vector (between
            # two points v1; I find the orthogonal vector to the plane b;
            # I find a second vector v2 orthogonal to v1 and on the plane
            # (<=> orthogonal to v1 and b); I normalize v1 and v2;
            # I get the components of the vectors w.r.t. v1 and v2
            # NOTE: there is at least 1 triangle => at least 3 points
            v1 = all_points_coords[1] - all_points_coords[0]
            temp_v2 = all_points_coords[2] - all_points_coords[0]
            b = np.cross(v1, temp_v2)
            v2 = np.cross(v1, b)
            # Normalize
            v1 = v1 / np.linalg.norm(v1)
            v2 = v2 / np.linalg.norm(v2)
            # get components
            x = [np.dot(point, v1) for point in all_points_coords]
            y = [np.dot(point, v2) for point in all_points_coords]
            # 2. do convexhull in 2D
            hull_face2d = ConvexHull(np.array([x, y]).T)
            # 3. get point indices of convex hull, convert back to original
            # 3D indices
            # 2dhull.vertices contains the segments, but with ids in the
            # smaller subset. We want the indices in the initial set:
            actual_points_idx = [
                all_points_idx[subset_idx] for subset_idx in hull_face2d.vertices
            ]
            # 4. add to faces list
            faces.append(
                [hull.points[point_idx].tolist() for point_idx in actual_points_idx]
            )

    ret_data["faces"] = faces
    return ret_data


if __name__ == "__main__":
    from pylab import figure, show
    from mpl_toolkits.mplot3d.art3d import Poly3DCollection

    # draw a vector
    from matplotlib.patches import FancyArrowPatch
    from mpl_toolkits.mplot3d import proj3d

    class Arrow3D(FancyArrowPatch):
        def __init__(self, xs, ys, zs, *args, **kwargs):
            FancyArrowPatch.__init__(self, (0, 0), (0, 0), *args, **kwargs)
            self._verts3d = xs, ys, zs

        def draw(self, renderer):
            xs3d, ys3d, zs3d = self._verts3d
            xs, ys, zs = proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
            self.set_positions((xs[0], ys[0]), (xs[1], ys[1]))
            FancyArrowPatch.draw(self, renderer)

    ##SC
    # faces_data = get_BZ(b1 = [1,0,0], b2 = [0,1,0], b3 = [0,0,1])
    ##BCC
    # faces_data = get_BZ(b1 = [1,1,0], b2 = [1,0,1], b3 = [0,1,1])
    ##FCC
    faces_data = get_BZ(b1=[1, 1, -1], b2=[1, -1, 1], b3=[-1, 1, 1])

    import json

    print(json.dumps(faces_data))

    faces_coords = faces_data["faces"]

    faces_count = defaultdict(int)
    for face in faces_coords:
        faces_count[len(face)] += 1

    for num_sides in sorted(faces_count.keys()):
        print("{} faces: {}".format(num_sides, faces_count[num_sides]))

    fig = figure()
    ax = fig.add_subplot(111, projection="3d")
    ax.add_collection3d(
        Poly3DCollection(
            faces_coords, linewidth=1, alpha=0.9, edgecolor="k", facecolor="#ccccff"
        )
    )

    # draw origin
    ax.scatter([0], [0], [0], color="g", s=100)

    axes_length = 2
    # Add axes
    ax.add_artist(
        Arrow3D(
            (0, axes_length),
            (0, 0),
            (0, 0),
            mutation_scale=20,
            lw=1,
            arrowstyle="-|>",
            color="k",
        )
    )
    ax.add_artist(
        Arrow3D(
            (0, 0),
            (0, axes_length),
            (0, 0),
            mutation_scale=20,
            lw=1,
            arrowstyle="-|>",
            color="k",
        )
    )
    ax.add_artist(
        Arrow3D(
            (0, 0),
            (0, 0),
            (0, axes_length),
            mutation_scale=20,
            lw=1,
            arrowstyle="-|>",
            color="k",
        )
    )

    ## Reset limits
    ax.set_xlim(-1, 1)
    ax.set_ylim(-1, 1)
    ax.set_zlim(-1, 1)
    ax.axis("off")
    ax.view_init(elev=0, azim=60)

    show()



def project_id(project_id):
    if (project_id==False):
        raise Exception("Define a project_id")
    
def job_id(job_id):
    if (job_id==False):
        raise Exception("Define a job_id")
    
def config(config):
    if config==False:
        raise Exception("Define a configuration")




"""
The seekpath.hpkot module contains routines to get automatically the
path in a 3D Brillouin zone to plot band structures according to the
HPKOT paper (see references below).

Author: Giovanni Pizzi, PSI

Licence: MIT License, see LICENSE.txt

.. note:: the list of point coordinates and example POSCAR files in
  the band_path_data subfolder have been provided by Yoyo Hinuma,
  Kyoto University, Japan. The POSCARs have been retrieved from
  the Materials Project (http://materialsproject.org).
"""


class EdgeCaseWarning(RuntimeWarning):
    """
    A warning issued when the cell is an edge case (e.g. orthorhombic
    symmetry, but ``a==b==c``.
    """

    pass


class SymmetryDetectionError(Exception):
    """
    Error raised if spglib could not detect the symmetry.
    """

    pass


def get_path(
    structure,
    with_time_reversal=True,
    threshold=1.0e-7,
    symprec=1e-05,
    angle_tolerance=-1.0,
):
    r"""
    Return the kpoint path information for band structure given a
    crystal structure, using the paths from the chosen recipe/reference.

    If you use this module, please cite the paper of the corresponding
    recipe (see parameter below).

    :param structure: The crystal structure for which we want to obtain
        the suggested path. It should be a tuple in the format
        accepted by spglib: ``(cell, positions, numbers)``, where
        (if N is the number of atoms):

        - ``cell`` is a :math:`3 \times 3` list of floats (``cell[0]`` is the first lattice
          vector, ...)
        - ``positions`` is a :math:`N \times 3` list of floats with the atomic coordinates
          in scaled coordinates (i.e., w.r.t. the cell vectors)
        - ``numbers`` is a length-:math:`N` list with integers identifying uniquely
          the atoms in the cell (e.g., the Z number of the atom, but
          any other positive non-zero integer will work - e.g. if you
          want to distinguish two Carbon atoms, you can set one number
          to 6 and the other to 1006)

    :param with_time_reversal: if False, and the group has no inversion
        symmetry, additional lines are returned as described in the HPKOT
        paper.

    :param recipe: choose the reference publication that defines the special
       points and paths.
       Currently, the following value is implemented:

       - ``hpkot``: HPKOT paper:
         Y. Hinuma, G. Pizzi, Y. Kumagai, F. Oba, I. Tanaka, Band structure
         diagram paths based on crystallography, Comp. Mat. Sci. 128, 140 (2017).
         DOI: 10.1016/j.commatsci.2016.10.015

    :param threshold: the threshold to use to verify if we are in
        and edge case (e.g., a tetragonal cell, but ``a==c``). For instance,
        in the tI lattice, if ``abs(a-c) < threshold``, a
        :py:exc:`~seekpath.hpkot.EdgeCaseWarning` is issued.
        Note that depending on the bravais lattice, the meaning of the
        threshold is different (angle, length, ...)

    :param symprec: the symmetry precision used internally by SPGLIB

    :param angle_tolerance: the angle_tolerance used internally by SPGLIB


    :return: a dictionary with the following
      keys:

        - ``point_coords``: a dictionary with label -> float coordinates
        - ``path``: a list of length-2 tuples, with the labels of the starting
          and ending point of each label section
        - ``has_inversion_symmetry``: True or False, depending on whether the
          input crystal structure has inversion symmetry or not.
        - ``augmented_path``: if True, it means that the path was
          augmented with the :math:`-k` points (this happens if both
          has_inversion_symmetry is False, and the user set
          with_time_reversal=False in the input)
        - ``bravais_lattice``: the Bravais lattice string (like ``cP``, ``tI``, ...)
        - ``bravais_lattice_extended``: the specific case used to define labels and
          coordinates (like ``cP1``, ``tI2``, ...)
        - ``cont_lattice``: three real-space vectors for the crystallographic
          conventional cell (``conv_lattice[0,:]`` is the first vector)
        - ``conv_positions``: fractional coordinates of atoms in the
          crystallographic conventional cell
        - ``conv_types``: list of integer types of the atoms in the crystallographic
          conventional cell (typically, the atomic numbers)
        - ``primitive_lattice``: three real-space vectors for the crystallographic
          primitive cell (``primitive_lattice[0,:]`` is the first vector)
        - ``primitive_positions``: fractional coordinates of atoms in the
          crystallographic primitive cell
        - ``primitive_types``: list of integer types of the atoms in the
          crystallographic primitive cell (typically, the atomic numbers)
        - ``reciprocal_primitive_lattice``: reciprocal-cell vectors for the
          primitive cell (vectors are rows: ``reciprocal_primitive_lattice[0,:]``
          is the first vector)
        - ``primitive_transformation_matrix``: the transformation matrix :math:`P` between
          the conventional and the primitive cell
        - ``inverse_primitive_transformation_matrix``: the inverse of the matrix :math:`P`
          (the determinant is integer and gives the ratio in volume between
          the conventional and primitive cells)
        - ``rotation_matrix``: rotation matrix in Cartesian space from the input
          cell to the standardized cell
        - ``volume_original_wrt_conv``: volume ratio of the user-provided cell
          with respect to the the crystallographic conventional cell
        - ``volume_original_wrt_prim``: volume ratio of the user-provided cell
          with respect to the the crystallographic primitive cell

    :note: An :py:exc:`~seekpath.hpkot.EdgeCaseWarning` is issued for
        edge cases (e.g. if ``a==b==c`` for
        orthorhombic systems). In this case, still one of the valid cases
        is picked.
    """
    import copy
    from math import sqrt
    import warnings

    import numpy as np

    from .tools import (
        check_spglib_version,
        extend_kparam,
        eval_expr,
        eval_expr_simple,
        get_cell_params,
        get_path_data,
        get_reciprocal_cell_rows,
        get_real_cell_from_reciprocal_rows,
    )
    from .spg_mapping import get_spgroup_data, get_primitive

    # I check if the SPGlib version is recent enough (raises ValueError)
    # otherwise
    spglib = check_spglib_version()

    structure_internal = (
        np.array(structure[0]),
        np.array(structure[1]),
        np.array(structure[2]),
    )

    # Symmetry analysis by SPGlib, get crystallographic lattice,
    # and cell parameters for this lattice
    dataset = spglib.get_symmetry_dataset(
        structure_internal, symprec=symprec, angle_tolerance=angle_tolerance
    )
    if dataset is None:
        raise SymmetryDetectionError(
            "Spglib could not detect the symmetry of the system"
        )
    conv_lattice = dataset["std_lattice"]
    conv_positions = dataset["std_positions"]
    conv_types = dataset["std_types"]
    a, b, c, cosalpha, cosbeta, cosgamma = get_cell_params(conv_lattice)
    spgrp_num = dataset["number"]
    # This is the transformation from the original to the crystallographic
    # conventional (called std in spglib)
    #  Lattice^{crystallographic_bravais} = L^{original} * transf_matrix
    transf_matrix = dataset["transformation_matrix"]
    volume_conv_wrt_original = np.linalg.det(transf_matrix)

    # Get the properties of the spacegroup, needed to get the bravais_lattice
    properties = get_spgroup_data()[spgrp_num]
    bravais_lattice = "{}{}".format(properties[0], properties[1])
    has_inv = properties[2]

    # Implement all different extended Bravais lattices
    if bravais_lattice == "cP":
        if spgrp_num >= 195 and spgrp_num <= 206:
            ext_bravais = "cP1"
        elif spgrp_num >= 207 and spgrp_num <= 230:
            ext_bravais = "cP2"
        else:
            raise ValueError(
                "Internal error! should be cP, but the "
                "spacegroup number is not in the correct range"
            )
    elif bravais_lattice == "cF":
        if spgrp_num >= 195 and spgrp_num <= 206:
            ext_bravais = "cF1"
        elif spgrp_num >= 207 and spgrp_num <= 230:
            ext_bravais = "cF2"
        else:
            raise ValueError(
                "Internal error! should be cF, but the "
                "spacegroup number is not in the correct range"
            )
    elif bravais_lattice == "cI":
        ext_bravais = "cI1"
    elif bravais_lattice == "tP":
        ext_bravais = "tP1"
    elif bravais_lattice == "tI":
        if abs(c - a) < threshold:
            warnings.warn("tI lattice, but a almost equal to c", EdgeCaseWarning)
        if c <= a:
            ext_bravais = "tI1"
        else:
            ext_bravais = "tI2"
    elif bravais_lattice == "oP":
        ext_bravais = "oP1"
    elif bravais_lattice == "oF":
        if abs(1.0 / (a**2) - (1.0 / (b**2) + 1.0 / (c**2))) < threshold:
            warnings.warn(
                "oF lattice, but 1/a^2 almost equal to 1/b^2 + 1/c^2", EdgeCaseWarning
            )
        if abs(1.0 / (c**2) - (1.0 / (a**2) + 1.0 / (b**2))) < threshold:
            warnings.warn(
                "oF lattice, but 1/c^2 almost equal to 1/a^2 + 1/b^2", EdgeCaseWarning
            )
        if 1.0 / (a**2) > 1.0 / (b**2) + 1.0 / (c**2):
            ext_bravais = "oF1"
        elif 1.0 / (c**2) > 1.0 / (a**2) + 1.0 / (b**2):
            ext_bravais = "oF2"
        else:  # 1/a^2, 1/b^2, 1/c^2 edges of a triangle
            ext_bravais = "oF3"
    elif bravais_lattice == "oI":
        # Sort a,b,c, first is the largest
        sorted_vectors = sorted([(c, 1, "c"), (b, 3, "b"), (a, 2, "a")])[::-1]
        if abs(sorted_vectors[0][0] - sorted_vectors[1][0]) < threshold:
            warnings.warn(
                "oI lattice, but the two longest vectors {} and {} "
                "have almost the same length".format(
                    sorted_vectors[0][2], sorted_vectors[1][2]
                ),
                EdgeCaseWarning,
            )
        ext_bravais = "{}{}".format(bravais_lattice, sorted_vectors[0][1])
    elif bravais_lattice == "oC":
        if abs(b - a) < threshold:
            warnings.warn("oC lattice, but a almost equal to b", EdgeCaseWarning)
        if a <= b:
            ext_bravais = "oC1"
        else:
            ext_bravais = "oC2"
    elif bravais_lattice == "oA":
        if abs(b - c) < threshold:
            warnings.warn("oA lattice, but b almost equal to c", EdgeCaseWarning)
        if b <= c:
            ext_bravais = "oA1"
        else:
            ext_bravais = "oA2"
    elif bravais_lattice == "hP":
        if spgrp_num in [
            143,
            144,
            145,
            146,
            147,
            148,
            149,
            151,
            153,
            157,
            159,
            160,
            161,
            162,
            163,
        ]:
            ext_bravais = "hP1"
        else:
            ext_bravais = "hP2"
    elif bravais_lattice == "hR":
        if abs(sqrt(3.0) * a - sqrt(2.0) * c) < threshold:
            warnings.warn(
                "hR lattice, but sqrt(3)a almost equal to sqrt(2)c", EdgeCaseWarning
            )
        if sqrt(3.0) * a <= sqrt(2.0) * c:
            ext_bravais = "hR1"
        else:
            ext_bravais = "hR2"
    elif bravais_lattice == "mP":
        ext_bravais = "mP1"
    elif bravais_lattice == "mC":
        if abs(b - a * sqrt(1.0 - cosbeta**2)) < threshold:
            warnings.warn(
                "mC lattice, but b almost equal to a*sin(beta)", EdgeCaseWarning
            )
        if b < a * sqrt(1.0 - cosbeta**2):
            ext_bravais = "mC1"
        else:
            if (
                abs(-a * cosbeta / c + a**2 * (1.0 - cosbeta**2) / b**2 - 1.0)
                < threshold
            ):
                warnings.warn(
                    "mC lattice, but -a*cos(beta)/c + "
                    "a^2*sin(beta)^2/b^2 almost equal to 1",
                    EdgeCaseWarning,
                )
            if -a * cosbeta / c + a**2 * (1.0 - cosbeta**2) / b**2 <= 1.0:
                # 12-face
                ext_bravais = "mC2"
            else:
                ext_bravais = "mC3"
    elif bravais_lattice == "aP":
        # First step: cell that is Niggli reduced in reciprocal space
        # I use the default eps here, this could be changed
        reciprocal_cell_orig = get_reciprocal_cell_rows(conv_lattice)
        ## This is Niggli-reduced
        reciprocal_cell2 = spglib.niggli_reduce(reciprocal_cell_orig)
        real_cell2 = get_real_cell_from_reciprocal_rows(reciprocal_cell2)
        # TODO: get transformation matrix?

        ka2, kb2, kc2, coskalpha2, coskbeta2, coskgamma2 = get_cell_params(
            reciprocal_cell2
        )

        conditions = np.array(
            [
                abs(kb2 * kc2 * coskalpha2),
                abs(kc2 * ka2 * coskbeta2),
                abs(ka2 * kb2 * coskgamma2),
            ]
        )
        M2_matrices = [
            np.array([[0, 0, 1], [1, 0, 0], [0, 1, 0]]),
            np.array([[0, 1, 0], [0, 0, 1], [1, 0, 0]]),
            np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]]),
        ]
        # TODO: manage edge cases
        smallest_condition = np.argsort(conditions)[0]
        M2 = M2_matrices[smallest_condition]
        # First change of vectors to have |ka3 kb3 cosgamma3| smallest
        real_cell3 = np.dot(np.array(real_cell2).T, M2).T
        reciprocal_cell3 = get_reciprocal_cell_rows(real_cell3)
        ka3, kb3, kc3, coskalpha3, coskbeta3, coskgamma3 = get_cell_params(
            reciprocal_cell3
        )
        if abs(coskalpha3) < threshold:
            warnings.warn(
                "aP lattice, but the k_alpha3 angle is almost equal " "to 90 degrees",
                EdgeCaseWarning,
            )
        if abs(coskbeta3) < threshold:
            warnings.warn(
                "aP lattice, but the k_beta3 angle is almost equal " "to 90 degrees",
                EdgeCaseWarning,
            )
        if abs(coskgamma3) < threshold:
            warnings.warn(
                "aP lattice, but the k_gamma3 angle is almost equal " "to 90 degrees",
                EdgeCaseWarning,
            )
        # Make them all-acute or all-obtuse with the additional conditions
        # explained in HPKOT
        # Note: cos > 0 => angle < 90deg
        if coskalpha3 > 0.0 and coskbeta3 > 0.0 and coskgamma3 > 0.0:  # 1a
            M3 = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]])
        elif coskalpha3 <= 0.0 and coskbeta3 <= 0.0 and coskgamma3 <= 0.0:  # 1b
            M3 = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]])
        elif coskalpha3 > 0.0 and coskbeta3 <= 0.0 and coskgamma3 <= 0.0:  # 2a
            M3 = np.array([[1, 0, 0], [0, -1, 0], [0, 0, -1]])
        elif coskalpha3 <= 0.0 and coskbeta3 > 0.0 and coskgamma3 > 0.0:  # 2b
            M3 = np.array([[1, 0, 0], [0, -1, 0], [0, 0, -1]])
        elif coskalpha3 <= 0.0 and coskbeta3 > 0.0 and coskgamma3 <= 0.0:  # 3a
            M3 = np.array([[-1, 0, 0], [0, 1, 0], [0, 0, -1]])
        elif coskalpha3 > 0.0 and coskbeta3 <= 0.0 and coskgamma3 > 0.0:  # 3b
            M3 = np.array([[-1, 0, 0], [0, 1, 0], [0, 0, -1]])
        elif coskalpha3 <= 0.0 and coskbeta3 <= 0.0 and coskgamma3 > 0.0:  # 4a
            M3 = np.array([[-1, 0, 0], [0, -1, 0], [0, 0, 1]])
        elif coskalpha3 > 0.0 and coskbeta3 > 0.0 and coskgamma3 <= 0.0:  # 4b
            M3 = np.array([[-1, 0, 0], [0, -1, 0], [0, 0, 1]])
        else:
            raise ValueError(
                "Problem identifying M3 matrix in aP lattice!"
                "Sign of cosines: cos(kalpha3){}0, "
                "cos(kbeta3){}0, cos(kgamma3){}0".format(
                    ">=" if coskalpha3 >= 0 else "<",
                    ">=" if coskbeta3 >= 0 else "<",
                    ">=" if coskgamma3 >= 0 else "<",
                )
            )

        real_cell_final = np.dot(real_cell3.T, M3).T
        reciprocal_cell_final = get_reciprocal_cell_rows(real_cell_final)
        ka, kb, kc, coskalpha, coskbeta, coskgamma = get_cell_params(
            reciprocal_cell_final
        )

        if coskalpha <= 0.0 and coskbeta <= 0.0 and coskgamma <= 0.0:
            # all-obtuse
            ext_bravais = "aP2"
        elif coskalpha >= 0.0 and coskbeta >= 0.0 and coskgamma >= 0.0:
            # all-acute
            ext_bravais = "aP3"
        else:
            raise ValueError(
                "Unexpected aP triclinic lattice, it neither "
                "all-obtuse nor all-acute! Sign of cosines: cos(kalpha){}0, "
                "cos(kbeta){}0, cos(kgamma){}0".format(
                    ">=" if coskalpha >= 0 else "<",
                    ">=" if coskbeta >= 0 else "<",
                    ">=" if coskgamma >= 0 else "<",
                )
            )

        # Get absolute positions
        conv_pos_abs = np.dot(conv_positions, conv_lattice)
        # Replace conv_lattice with the new conv_lattice
        conv_lattice = np.array(real_cell_final)
        # Store the relative coords with respect to the new vectors
        # TODO: decide if we want to do %1. for the fractional coordinates
        conv_positions = np.dot(conv_pos_abs, np.linalg.inv(conv_lattice))
        # TODO: implement the correct one (probably we need the matrix
        # out from niggli, and then we can combine it with M2 and M3??)
        # We set it to None for the time being to avoid confusion
        transformation_matrix = None

    else:
        raise ValueError(
            "Unknown type '{}' for spacegroup {}".format(
                bravais_lattice, dataset["number"]
            )
        )

    # NOTE: we simply use spglib.find_primitive, because the
    # find_primitive of spglib follows a different convention for mC
    # and oA as explained in the HPKOT paper
    (prim_lattice, prim_pos, prim_types), (P, invP), conv_prim_mapping = get_primitive(
        structure=(conv_lattice, conv_positions, conv_types),
        bravais_lattice=bravais_lattice,
    )

    # Get the path data (k-parameters definitions, defition of the points,
    # suggested path)
    kparam_def, points_def, path = get_path_data(ext_bravais)

    # Get the actual numerical values of the k-parameters
    # Note: at each step, I pass kparam and store the new
    # parameter in the same dictionary. This allows to have
    # some parameters defined implicitly in terms of previous
    # parameters, as far as they are returned in the
    kparam = {}
    for kparam_name, kparam_expr in kparam_def:
        kparam[kparam_name] = eval_expr(
            kparam_expr, a, b, c, cosalpha, cosbeta, cosgamma, kparam
        )

    # Extend kparam with additional simple expressions (like 1-a, ...)
    kparam_extended = extend_kparam(kparam)

    # Now I have evaluated all needed kparams; I can compute the actual
    # coordinates of the relevant kpoints, using eval_expr_simple
    points = {}
    for pointname, coords_def in points_def.items():
        coords = [eval_expr_simple(_, kparam_extended) for _ in coords_def]
        points[pointname] = coords

    # If there is no inversion symmetry nor time-reversal symmetry, add
    # additional path
    if not has_inv and not with_time_reversal:
        augmented_path = True
    else:
        augmented_path = False

    if augmented_path:
        for pointname, coords in list(points.items()):
            if pointname == "GAMMA":
                continue
            points["{}'".format(pointname)] = [-coords[0], -coords[1], -coords[2]]
        old_path = copy.deepcopy(path)
        for start_p, end_p in old_path:
            if start_p == "GAMMA":
                new_start_p = start_p
            else:
                new_start_p = "{}'".format(start_p)
            if end_p == "GAMMA":
                new_end_p = end_p
            else:
                new_end_p = "{}'".format(end_p)
            path.append((new_start_p, new_end_p))

    return {
        "point_coords": points,
        "path": path,
        "has_inversion_symmetry": has_inv,
        "augmented_path": augmented_path,
        "bravais_lattice": bravais_lattice,
        "bravais_lattice_extended": ext_bravais,
        "conv_lattice": conv_lattice,
        "conv_positions": conv_positions,
        "conv_types": conv_types,
        "primitive_lattice": prim_lattice,
        "primitive_positions": prim_pos,
        "primitive_types": prim_types,
        "reciprocal_primitive_lattice": get_reciprocal_cell_rows(prim_lattice),
        # The following: between conv and primitive, see docstring of
        # spg_mapping.get_P_matrix
        "inverse_primitive_transformation_matrix": invP,
        "primitive_transformation_matrix": P,
        # For the time being disabled, not valid for aP lattices
        # (for which we would need the transformation matrix from niggli)
        #'transformation_matrix': transf_matrix,
        "volume_original_wrt_conv": volume_conv_wrt_original,
        "volume_original_wrt_prim": volume_conv_wrt_original * np.linalg.det(invP),
        "spacegroup_number": dataset["number"],
        "spacegroup_international": dataset["international"],
        "rotation_matrix": dataset["std_rotation_matrix"],
    }




import os
import numpy as np
import matplotlib.pyplot as plt
out_files = os.listdir('./results/')

total_energy = []
e_fermi = []
for prefix in out_files:
    temp_file = open(f'./results/{prefix}/scf.out', 'r').readlines()
    for i in range(len(temp_file)):
        if len(temp_file[i].split())>2:
            if temp_file[i].split()[0]=='!':
                temp_en=temp_file[i].split()[4]
            # if temp_file[i].split()[0]=='the':
            #     temp_fermi=temp_file[i].split()[-2]
    total_energy.append([float(temp_en),float(prefix)])
    # e_fermi.append([float(temp_fermi),float(file[:-13])])
    
tot_en = np.array(total_energy)
tot_en = tot_en[tot_en[:, 1].argsort()]


# e_f = np.array(e_fermi)
# e_f = e_f[e_f[:, 1].argsort()]

# fig = plt.figure(figsize=(8,6))
plt.plot(tot_en.T[1],tot_en.T[0])
# plt.ylabel('Total Energy [Ry]',fontsize=20)
# plt.xlabel('Smearing Value [Ry]',fontsize=20)
# plt.xticks(fontsize=20)
# plt.yticks(fontsize=20)
plt.savefig('total_sigma.png')



"""Various utilities."""
import numpy
import numpy.linalg
from math import sqrt


def eval_expr_simple(expr, kparam):  # pylint=disable: too-many-return-statements
    """
    To evaluate expressions tha only require kparams and not a, b, c, ...
    """
    if expr == "0":
        return 0.0
    if expr == "1/2":
        return 1.0 / 2.0
    if expr == "1":
        return 1.0
    if expr == "-1/2":
        return -1.0 / 2.0
    if expr == "1/4":
        return 1.0 / 4.0
    if expr == "3/8":
        return 3.0 / 8.0
    if expr == "3/4":
        return 3.0 / 4.0
    if expr == "5/8":
        return 5.0 / 8.0
    if expr == "1/3":
        return 1.0 / 3.0
    try:
        return kparam[expr]
    except KeyError as exc:
        raise ValueError(
            "Asking for evaluation of symbol '{}' in "
            "eval_expr_simple but this has not been defined or not "
            "yet computed".format(str(exc))
        )


def extend_kparam(kparam):
    """
    Extend the list of kparam with also expressions like :math:`1-x`, ...

    :param kparam: a dictionary where the key is the expression as a string and
       the value is the numerical value
    :return: a similar dictionary, extended with simple expressions
    """
    kparam_extended = {}
    for key, val in kparam.items():
        kparam_extended[key] = val
        kparam_extended["-{}".format(key)] = -val
        kparam_extended["1-{}".format(key)] = 1.0 - val
        kparam_extended["-1+{}".format(key)] = -1.0 + val
        kparam_extended["1/2-{}".format(key)] = 1.0 / 2.0 - val
        kparam_extended["1/2+{}".format(key)] = 1.0 / 2.0 + val

    return kparam_extended


def eval_expr(  # pylint: disable=too-many-return-statements,unused-argument
    expr, a, b, c, cosalpha, cosbeta, cosgamma, kparam
):
    r"""
    Given a string expression as a function of the parameters ``a``, ``b``, ``c`` (lengths of the
    cell lattice vectors) and ``cosalpha``, ``cosbeta``, ``cosgamma`` (the cosines of the three
    angles between lattice vectors) returns the numerical value of the expression.

    :param a: length of the first lattice vector
    :param b: length of the second lattice vector
    :param c: length of the third lattice vector
    :param cosalpha: cosine of the :math:`\alpha` angle (between lattice vectors 2 and 3)
    :param cosbeta: cosine of the :math:`\beta` angle (between lattice vectors 1 and 3)
    :param cosgamma: cosine of the :math:`\gamma` angle (between lattice vectors 1 and 2)
    :param kparam: a dictionary that associates the value to expressions as a function
        of the ``a, b, c, cosalpha, cosbeta, cosgamma`` parameters

    :return: the value of the expression for the given values of the cell parameters

    .. note::  To evaluate expressions, I hardcode a table of existing expressions in the
        DB rather than parsing the string (to avoid additional dependencies and
        avoid the use of ``eval``).
    """
    from math import sqrt

    # sinalpha = sqrt(1.0 - cosalpha ** 2)
    sinbeta = sqrt(1.0 - cosbeta**2)
    # singamma = sqrt(1.0 - cosgamma ** 2)

    try:
        if expr == "(a*a/b/b+(1+a/c*cosbeta)/sinbeta/sinbeta)/4":
            return (a * a / b / b + (1.0 + a / c * cosbeta) / sinbeta / sinbeta) / 4.0
        if expr == "1-Z*b*b/a/a":
            Z = kparam["Z"]
            return 1.0 - Z * b * b / a / a
        if expr == "1/2-2*Z*c*cosbeta/a":
            Z = kparam["Z"]
            return 1.0 / 2.0 - 2.0 * Z * c * cosbeta / a
        if expr == "E/2+a*a/4/b/b+a*c*cosbeta/2/b/b":
            E = kparam["E"]
            return E / 2.0 + a * a / 4.0 / b / b + a * c * cosbeta / 2.0 / b / b
        if expr == "2*F-Z":
            F = kparam["F"]
            Z = kparam["Z"]
            return 2.0 * F - Z
        if expr == "c/2/a/cosbeta*(1-4*U+a*a*sinbeta*sinbeta/b/b)":
            U = kparam["U"]
            return (
                c
                / 2.0
                / a
                / cosbeta
                * (1.0 - 4.0 * U + a * a * sinbeta * sinbeta / b / b)
            )
        if expr == "-1/4+W/2-Z*c*cosbeta/a":
            W = kparam["W"]
            Z = kparam["Z"]
            return -1.0 / 4.0 + W / 2.0 - Z * c * cosbeta / a
        if expr == "(2+a/c*cosbeta)/4/sinbeta/sinbeta":
            return (2.0 + a / c * cosbeta) / 4.0 / sinbeta / sinbeta
        if expr == "3/4-b*b/4/a/a/sinbeta/sinbeta":
            return 3.0 / 4.0 - b * b / 4.0 / a / a / sinbeta / sinbeta
        if expr == "S-(3/4-S)*a*cosbeta/c":
            S = kparam["S"]
            return S - (3.0 / 4.0 - S) * a * cosbeta / c
        if expr == "(1+a*a/b/b)/4":
            return (1.0 + a * a / b / b) / 4.0
        if expr == "-a*c*cosbeta/2/b/b":
            return -a * c * cosbeta / 2.0 / b / b
        if expr == "1+Z-2*M":
            Z = kparam["Z"]
            M = kparam["M"]
            return 1.0 + Z - 2.0 * M
        if expr == "X-2*D":
            X = kparam["X"]
            D = kparam["D"]
            return X - 2 * D
        if expr == "(1+a/c*cosbeta)/2/sinbeta/sinbeta":
            return (1.0 + a / c * cosbeta) / 2.0 / sinbeta / sinbeta
        if expr == "1/2+Y*c*cosbeta/a":
            Y = kparam["Y"]
            return 1.0 / 2.0 + Y * c * cosbeta / a
        if expr == "a*a/4/c/c":
            return a * a / 4.0 / c / c
        if expr == "5/6-2*D":
            D = kparam["D"]
            return 5.0 / 6.0 - 2.0 * D
        if expr == "1/3+D":
            D = kparam["D"]
            return 1.0 / 3.0 + D
        if expr == "1/6-c*c/9/a/a":
            return 1.0 / 6.0 - c * c / 9.0 / a / a
        if expr == "1/2-2*Z":
            Z = kparam["Z"]
            return 1.0 / 2.0 - 2.0 * Z
        if expr == "1/2+Z":
            Z = kparam["Z"]
            return 1.0 / 2.0 + Z
        if expr == "(1+b*b/c/c)/4":
            return (1.0 + b * b / c / c) / 4.0
        if expr == "(1+c*c/b/b)/4":
            return (1.0 + c * c / b / b) / 4.0
        if expr == "(1+b*b/a/a)/4":
            return (1.0 + b * b / a / a) / 4.0
        if expr == "(1+a*a/b/b-a*a/c/c)/4":
            return (1.0 + a * a / b / b - a * a / c / c) / 4.0
        if expr == "(1+a*a/b/b+a*a/c/c)/4":
            return (1.0 + a * a / b / b + a * a / c / c) / 4.0
        if expr == "(1+c*c/a/a-c*c/b/b)/4":
            return (1.0 + c * c / a / a - c * c / b / b) / 4.0
        if expr == "(1+c*c/a/a+c*c/b/b)/4":
            return (1.0 + c * c / a / a + c * c / b / b) / 4.0
        if expr == "(1+b*b/a/a-b*b/c/c)/4":
            return (1.0 + b * b / a / a - b * b / c / c) / 4.0
        if expr == "(1+c*c/b/b-c*c/a/a)/4":
            return (1.0 + c * c / b / b - c * c / a / a) / 4.0
        if expr == "(1+a*a/c/c)/4":
            return (1.0 + a * a / c / c) / 4.0
        if expr == "(b*b-a*a)/4/c/c":
            return (b * b - a * a) / 4.0 / c / c
        if expr == "(a*a+b*b)/4/c/c":
            return (a * a + b * b) / 4.0 / c / c
        if expr == "(1+c*c/a/a)/4":
            return (1.0 + c * c / a / a) / 4.0
        if expr == "(c*c-b*b)/4/a/a":
            return (c * c - b * b) / 4.0 / a / a
        if expr == "(b*b+c*c)/4/a/a":
            return (b * b + c * c) / 4.0 / a / a
        if expr == "(a*a-c*c)/4/b/b":
            return (a * a - c * c) / 4.0 / b / b
        if expr == "(c*c+a*a)/4/b/b":
            return (c * c + a * a) / 4.0 / b / b
        if expr == "a*a/2/c/c":
            return a * a / 2.0 / c / c
        raise ValueError(
            "Unknown expression, define a new case:\n"
            '        elif expr == "{0}":\n'
            "            return {0}".format(expr)
        )
    except KeyError as exc:
        raise ValueError(
            "Asking for evaluation of symbol '{}' but this has "
            "not been defined or not yet computed".format(str(exc))
        )


def check_spglib_version():
    """
    Check the SPGLIB version and raise a ValueError if the version is
    older than 1.9.4.

    Also raises an warning if the user has a version of SPGLIB that is
    older than 1.13, because before then there were some bugs (e.g.
    wrong treatment of oI, see e.g. issue )

    Return the spglib module.
    """
    try:
        import spglib
    except ImportError:
        raise ValueError(
            "spglib >= 1.9.4 is required for the creation "
            "of the k-paths, but it could not be imported"
        )

    try:
        version = spglib.__version__
    except NameError:
        version = "1.8.0"  # or older, version was introduced only recently

    try:
        version_pieces = [int(_) for _ in version.split(".")]
        if len(version_pieces) < 3:
            raise ValueError
    except ValueError:
        raise ValueError("Unable to parse version number")

    if tuple(version_pieces[:2]) < (1, 9):
        raise ValueError("Invalid spglib version, need >= 1.9.4")

    if version_pieces[:2] == (1, 9) and version_pieces[2] < 4:
        raise ValueError("Invalid spglib version, need >= 1.9.4")

    if tuple(version_pieces[:2]) < (1, 13):
        import warnings

        warnings.warn(
            "You have a version of SPGLIB older than 1.13, "
            "please consider upgrading to 1.13 or later since some bugs "
            "have been fixed",
            RuntimeWarning,
        )

    return spglib


def get_cell_params(cell):
    r"""
    Return (a,b,c,cosalpha,cosbeta,cosgamma) given a :math:`3\times 3` cell

    .. note:: Rows are vectors: ``v1 = cell[0]``, ``v2 = cell[1]``, ``v3 = cell[3]``
    """
    v1, v2, v3 = numpy.array(cell)
    a = sqrt(sum(v1**2))
    b = sqrt(sum(v2**2))
    c = sqrt(sum(v3**2))
    cosalpha = numpy.dot(v2, v3) / b / c
    cosbeta = numpy.dot(v1, v3) / a / c
    cosgamma = numpy.dot(v1, v2) / a / b

    return (a, b, c, cosalpha, cosbeta, cosgamma)


def get_reciprocal_cell_rows(real_space_cell):
    r"""
    Given the cell in real space (3x3 matrix, vectors as rows,
    return the reciprocal-space cell where again the G vectors are
    rows, i.e. satisfying
    ``dot(real_space_cell, reciprocal_space_cell.T)`` = :math:`2 \pi I`,
    where :math:`I` is the :math:`3\times 3` identity matrix.

    :return: the :math:`3\times 3` list of reciprocal lattice vectors where each row is
        one vector.
    """
    reciprocal_space_columns = 2.0 * numpy.pi * numpy.linalg.inv(real_space_cell)
    return (reciprocal_space_columns.T).tolist()


def get_real_cell_from_reciprocal_rows(reciprocal_space_rows):
    r"""
    Given the cell in reciprocal space (3x3 matrix, G vectors as rows,
    return the real-space cell where again the R vectors are
    rows, i.e. satisfying
    ``dot(real_space_cell, reciprocal_space_cell.T)`` = :math:`2 \pi I`,
    where :math:`I` is the :math:`3\times 3` identity matrix.

    .. note::  This is actually the same as :py:func:`get_reciprocal_cell_rows`.

    :return: the :math:`3\times 3` list of real lattice vectors where each row is
        one vector.
    """
    real_space_columns = 2.0 * numpy.pi * numpy.linalg.inv(reciprocal_space_rows)
    return (real_space_columns.T).tolist()


def get_path_data(ext_bravais):
    """
    Given an extended Bravais symbol among those defined in the HPKOT paper
    (only first three characters, like cF1), return the points and the
    suggested path.

    :param ext_bravais: a string among the allowed etended Bravais lattices
        defined in HPKOT.
    :return: a tuple ``(kparam_def, points_def, path)`` where the
        first element is the list with the definition of the
        k-point parameters, the second is the dictionary with the
        definition of the k-points, and the third is the list
        with the suggested paths.

    .. note:: ``kparam_def`` has to be a list and not a dictionary
        because the order matters (later k-parameters can be defined
        in terms of previous ones)
    """
    import os

    # Get the data from the band_data folder
    this_folder = os.path.split(os.path.abspath(__file__))[0]
    folder = os.path.join(this_folder, "band_path_data", ext_bravais)
    path_file = os.path.join(folder, "path.txt")
    points_file = os.path.join(folder, "points.txt")
    kparam_file = os.path.join(folder, "k_vector_parameters.txt")
    with open(kparam_file) as f:
        kparam_raw = [_.split() for _ in f.readlines() if _.strip()]
    with open(points_file) as f:
        points_raw = [_.split() for _ in f.readlines()]
    with open(path_file) as f:
        path_raw = [_.split() for _ in f.readlines()]

    # check
    if any(len(_) != 2 for _ in kparam_raw):
        raise ValueError("Invalid line length in {}".format(kparam_file))
    if any(len(_) != 2 for _ in path_raw):
        raise ValueError("Invalid line length in {}".format(path_file))
    if any(len(_) != 4 for _ in points_raw):
        raise ValueError("Invalid line length in {}".format(points_file))
    # order must be preserved here
    kparam_def = [(_[0], _[1].strip()) for _ in kparam_raw]
    points_def = {}
    for label, kPx, kPy, kPz in points_raw:
        if label in points_def:
            raise ValueError(
                "Internal error! Point {} defined multiple times "
                "for Bravais lattice {}".format(label, ext_bravais)
            )
        points_def[label] = (kPx, kPy, kPz)
    path = [(_[0], _[1]) for _ in path_raw]

    # check path is valid
    for p1, p2 in path:
        if p1 not in points_def:
            raise ValueError(
                "Point {} found in path (for {}) but undefined!".format(p1, ext_bravais)
            )
        if p2 not in points_def:
            raise ValueError(
                "Point {} found in path (for {}) but undefined!".format(p2, ext_bravais)
            )

    return (kparam_def, points_def, path)



def check_relax(path):
    lines = open(path, 'r').readlines()
    for i in lines:
        if "bfgs failed after" in i:
            return False
        if "bfgs converged" in i:
            return True


import numpy as np
import matplotlib.pyplot as plt
import glob
from . import reads
from . import utils


def plot(self,calculation,save,xlim=False,ylim=False):
    if calculation=='electron':
        plot_electron(self,ylim=ylim,save=save)
    if calculation=='phonon':
        plot_phonon(self,save=save)
    if calculation=='dos':
        plot_dos(self,xlim=xlim,save=save)
    if calculation=='pdos':
        plot_pdos(self,xlim=xlim,save=save)
    if calculation=='kdos':
        plot_kdos(self,ylim=ylim,save=save)

def plot_electron(self,ylim=False,show=False,save=True):
    sym = reads.read_symmetries(f'./Projects/{self.project_id}/{self.job_id}/bands-pp.out')
    fermi = reads.read_efermi(f'./Projects/{self.project_id}/{self.job_id}/scf.out')
    fig = plt.figure(figsize=(8,6))
    data = np.loadtxt(f'./Projects/{self.project_id}/{self.job_id}/bands.dat.gnu')
    k = np.unique(data[:, 0])
    bands = np.reshape(data[:, 1], (-1, len(k)))
    for band in range(len(bands)):
        plt.plot(k, bands[band, :],c='black')
    plt.xticks(sym,self.path)
    for i in range(1,len(sym)-1):
        plt.axvline(sym[i],c='black')
    plt.axhline(float(fermi),c='red')
    plt.text(-0.2, float(fermi), r'$\epsilon_{Fermi}$',color='red')
    plt.ylim(ylim[0],ylim[1])
    plt.xlim(sym[0],sym[-1])
    if save==True:
        plt.savefig(f'./Projects/{self.project_id}/{self.job_id}/band.png')
    if show==True:
        return fig
    

def plot_phonon(self,save=True):
    sym = []
    point = [0]
    for k,i in enumerate(self.config['pw']['k_points_bands']):
        sym.append(i['label'].split()[1])
        if k!=len(self.config['pw']['k_points_bands'])-1:
            point.append(point[k]+int(i['number']))
    freq = np.loadtxt(f"./Projects/{self.project_id}/{self.job_id}/{self.job_id}.freq.gp")
    ph_path = freq.T[0]/max(freq.T[0])
    cm2mev = 0.124
    fig = plt.figure(figsize=(7,6))
    
    for i in range(1,len(freq.T)):
        plt.plot(ph_path,freq.T[i]*cm2mev,linewidth=2,color="blue")
    # print(point)
    tick = [ ph_path[i] for i in point ]
    for i in tick[1:-1]:
        plt.axvline(i,linestyle="--",color="black")
    plt.xticks(tick,sym,fontsize=15)
    plt.ylim(0,)
    plt.xlim(0,ph_path[-1])
    plt.ylabel("ω (meV)",fontsize=15)
    if save==True:
        plt.savefig(f'./Projects/{self.project_id}/{self.job_id}/phonon_band.png')
    plt.show()
    
def plot_dos(self,save=True,xlim=False):
    try :
        self.magnetic_order
        fermi = reads.read_efermi(f'./Projects/{self.project_id}/{self.job_id}/nscf.out')
        # load data
        energy, up, down, int_dos = np.loadtxt(f'./Projects/{self.project_id}/{self.job_id}/dos.dat', unpack=True)
        # make plot
        plt.figure(figsize = (12, 6))
        plt.plot(energy, up, linewidth=0.75, color='red')
        plt.plot(energy, -down, linewidth=0.75, color='blue')
        plt.yticks([])
        plt.xlabel('Energy (eV)')
        plt.ylabel('DOS')
        plt.axvline(x=fermi, linewidth=0.5, color='k', linestyle=(0, (8, 10)))
        plt.xlim(10, 30)
        # plt.ylim(0, )
        plt.fill_between(energy, 0, -down, where=(energy < fermi), facecolor='red', alpha=0.25)
        plt.fill_between(energy, 0, up, where=(energy < fermi), facecolor='red', alpha=0.25)
        plt.text(fermi+1, up.mean(), 'Fermi energy', rotation=90)
        plt.title(self.job_id.upper())
        if save==True:
            plt.savefig(f'./Projects/{self.project_id}/{self.job_id}/dos.png')
    except:
        fermi = reads.read_efermi(f'./Projects/{self.project_id}/{self.job_id}/nscf.out')
        # load data
        energy, dos, idos = np.loadtxt(f'./Projects/{self.project_id}/{self.job_id}/dos.dat', unpack=True)
        # make plot
        plt.figure(figsize = (12, 6))
        plt.plot(energy, dos, linewidth=0.75, color='red')
        plt.yticks([])
        plt.xlabel('Energy (eV)')
        plt.ylabel('DOS')
        plt.axvline(x=fermi, linewidth=0.5, color='k', linestyle=(0, (8, 10)))
        if xlim:
            plt.xlim(xlim)
        else:
            plt.xlim(-20, 20)
        plt.ylim(0, )
        plt.fill_between(energy, 0, dos, where=(energy < fermi), facecolor='red', alpha=0.25)
        plt.text(fermi+1, dos.mean(), 'Fermi energy', rotation=90)
        if save==True:
            plt.savefig(f'./Projects/{self.project_id}/{self.job_id}/dos.png')
    
def plot_pdos(self,xlim=False,save=False):
    files = glob.glob(f'./Projects/{self.project_id}/{self.job_id}/sumpdos*')
    files.append(f'./Projects/{self.project_id}/{self.job_id}/projwfc.dat.pdos_tot')
    fermi = reads.read_efermi(f'./Projects/{self.project_id}/{self.job_id}/nscf.out')
    plt.figure(figsize = (8, 4))
    for i in files:
        parts = i.split("_")
        if len(parts)==2:
            label='Total'
        else:
            atom = parts[1]
            orbital = parts[2].split('.')[0]
            label=atom+'-'+orbital
        energy, pdos = utils.pdos_loader(i)
        plt.plot(energy, pdos, linewidth=0.75,label=label)
        plt.yticks([])
        plt.xlabel('Energy (eV)')
        plt.ylabel('DOS')
        plt.axvline(x= fermi, linewidth=0.5, color='k', linestyle=(0, (8, 10)))
        if xlim:
            plt.xlim(xlim)
        else:
            plt.xlim(-20, 20)
        if label=='Total':
            plt.ylim(0, max(pdos)*1.2)
        plt.fill_between(energy, 0, pdos, where=(energy < fermi), alpha=0.25)
    plt.text(fermi+1, pdos.mean(), 'Fermi energy', rotation=90)
    plt.legend(frameon=False)
    plt.savefig(f'./Projects/{self.project_id}/{self.job_id}/pdos.png')

def plot_kdos(self,ylim=False,save=False):
    files = glob.glob(f'./Projects/{self.project_id}/{self.job_id}/sumkdos*')
    files.append(f'./Projects/{self.project_id}/{self.job_id}/dos.k.pdos_tot')
    fermi = reads.read_efermi(f'./Projects/{self.project_id}/{self.job_id}/scf.out')
    for i in files:
        parts = i.split("_")
        if len(parts)==2:
            label='Total'
        else:
            atom = parts[1]
            orbital = parts[2].split('.')[0]
            label=atom+'-'+orbital
        # print(label)
        data = np.loadtxt(i)

        k = np.unique(data[:, 0])  # k values
        e = np.unique(data[:, 1])  # dos energy values

        dos = np.zeros([len(k), len(e)])

        for i in range(len(data)):
            e_index = int(i % len(e))
            k_index = int(data[i][0] - 1)
            dos[k_index, e_index] = data[i][2]
        sym = reads.read_symmetries(f'./Projects/{self.project_id}/{self.job_id}/bands-pp.out')
        sym =sym/max(sym)*max(k)
        fermi = reads.read_efermi(f'./Projects/{self.project_id}/{self.job_id}/scf.out')
        plt.pcolormesh(k, e, dos.T, cmap='magma', shading='auto')
        plt.xticks(sym,self.path)
        for i in range(1,len(sym)-1):
            plt.axvline(sym[i],c='white')
        plt.axhline(float(fermi),c='white')
        if ylim:
            plt.ylim(ylim)
        plt.ylabel('Energy (eV)')
        plt.title(label)
        plt.savefig(f'./Projects/{self.project_id}/{self.job_id}/kdos_{label}.png')


from . import utils
from . import reads
from . import generate
from . import compute
from . import kpoints
from . import plots
from . import structure
from . import writes
from . import checks
import numpy as np 

class project:
    def __init__(self,project_id,config=False):
        self.project_id = project_id
        self.config = utils.configure(config)
        self.atom = False
        self.lattice = False
        self.calculation=False
        self.job_id = 'results'
        self.path = False
        self.poscar= False
        self.debug = False
        self.magnetism=False
        self.gpu = False
        self.num_core=1

    def set_cores(self,value):
        self.num_core=value

    def energy(self):
        return utils.get_total_energy(self)

    def from_poscar(self,directory=False):
        if directory==False:
            self.poscar=f'./Structures/{self.project_id}.poscar'
            lattice, atom = reads.read_poscar(self.poscar)
        else:
            self.poscar=directory
            lattice, atom = reads.read_poscar(directory)
        self.config['pw']['atomic_positions'] = atom
        self.config['pw']['cell_parameters'] = lattice
    
    def from_json(self,data=False,path=False):
        lattice, atom = reads.read_json(data=data,path=path)
        self.config['pw']['atomic_positions'] = atom
        self.config['pw']['cell_parameters'] = lattice
    
    def set_calculation(self,calculation_type):
        self.calculation = calculation_type
        if calculation_type in ['vc-relax','relax','scf','nscf','bands']:
            self.package='pw'
        elif calculation_type == 'bands-pp':
            self.package='bands'
        elif calculation_type in ['pdos','kdos']:
            self.package='projwfc'
        else:
            self.package=calculation_type

        self.file_path = f"./Projects/{self.project_id}/{self.job_id}/{self.calculation}.in"

    def hubbard(self,atom,orbital,value,interaction='U',projection="atomic"):
        self.config['pw']['hubbard']['projection']=projection
        self.config['pw']['hubbard']['terms'].append({"interaction":interaction,'atom':atom,'orbital':orbital,'value':value})
    

    def set_pseudo(self,pseudo_type=False,path=False):
        if path is not False:
            self.config['pw']['control']['pseudo_dir']=f'{path}'
        elif pseudo_type is not False:
            self.config['pw']['control']['pseudo_dir']='./pseudos/'+pseudo_type.upper()+'/'
    
    def make_afm(self,magnetic_atom,angle1=False,angle2=False):
        afm_models = utils.afm_maker(self,magnetic_atom,angle1=angle1,angle2=angle2)
        return afm_models
    def make_fm(self,magnetic_atom,angle1=False,angle2=False):
        fm = utils.fm_maker(self,magnetic_atom,angle1=angle1,angle2=angle2)
        return fm
    def magnetize(self,magnetic_atom,angle1=False,angle2=False):
        fm = utils.fm_maker(self,magnetic_atom,angle1=angle1,angle2=angle2)
        afm = utils.afm_maker(self,magnetic_atom,angle1=angle1,angle2=angle2)
        models = [fm,*afm]
        return models

    def band_points(self,path,number,file_path=False,file_name=False,file_format=False):
        self.path=path
        file_path = f'./Projects/{self.project_id}/{self.job_id}'
        file_name = f"{self.project_id}_{self.job_id}"
        self.export_structure(file_path=file_path,file_name=file_name)
        points = self.get_points(file_path=f'{file_path}/{file_name}.poscar',file_format="vasp-ase")
        k_path = kpoints.band_input(path,points,number)
        self.config['pw']['k_points_bands'] = k_path



    def occupations(self,occupation):
        self.config['pw']['system']['occupations'] = occupation
    def ecutwfc(self,number):
        self.config['pw']['system']['ecutwfc'] = number
    def ecutrho(self,number):
        self.config['pw']['system']['ecutrho'] = number
    def conv_thr(self,number):
        self.config['pw']['electrons']['conv_thr']=number
    def ph_thr(self,number):
        self.config['ph']['inputph']['tr2_ph']=number
    def force(self,value=True):
        self.config['pw']['control']['tprnfor'] = value
    def stress(self,value=True):
        self.config['pw']['control']['tstress'] = value
    def etot_conv_thr(self,value):
        self.config['pw']['control']['etot_conv_thr'] = value
    def forc_conv_thr(self,value):
        self.config['pw']['control']['forc_conv_thr'] = value
    def make_layer(self,layer_type='mono',direction='z'):
        if layer_type=='mono':
            self.config['pw']['atomic_positions'] = utils.make_monolayer(self.config['pw']['atomic_positions'],direction)
    def k_points(self,number):
        if type(number) == int :
            self.config['pw']['k_points']=f'{number} {number} {number} 0 0 0'
        elif len(number) == 3:
            self.config['pw']['k_points']=f'{number[0]} {number[1]} {number[2]} 0 0 0'
        else:
            raise Exception("K points can be either a number or an array with 3 enteries")
        
    def atoms(self):
        return self.config['pw']['atomic_positions']
    
    def positions(self):
        atom =  np.array(self.config['pw']['atomic_positions']).T[1:].T.astype(float)
        return atom
    def cell(self):
        return self.config['pw']['cell_parameters']
    
    def add_vacuum(self,direction,vector):
            cell = np.array(self.config['pw']["cell_parameters"],dtype=float)
            atom =  self.positions()
            vacuum_atom, vacuum_cell =  utils.shift_cell(atom,cell,direction,vector)
            for i,atom in enumerate(vacuum_atom):
                for j in range(3):
                    self.config['pw']['atomic_positions'][i][1+j]=atom[j].astype(str)
            self.config['pw']["cell_parameters"] = vacuum_cell


    def shift_atoms(self,vector):
        atoms = np.array(self.config['pw']['atomic_positions']).T[1:].T.astype(float)
        shifted = utils.shift_frac(atoms=atoms,vector=vector)
        for i,atom in enumerate(shifted):
            for j in range(3):
                self.config['pw']['atomic_positions'][i][1+j]=atom[j].astype(str)



        self.config['pw']['atomic_positions']


    def plot(self,calculation,save=False,xlim=False,ylim=False):
        plots.plot(self,calculation,save,xlim,ylim)

    def nbnd(self,number):
        self.config['pw']['system']['nbnd']=number
    
    def get_nbnd(self):
        return int(reads.read_num_bands(self))
    
    def mixing_mode(self,value):
        self.config['pw']['electrons']['mixing_mode']=value
    def smearing(self,value):
        self.config['pw']['system']['smearing'] = value
    def degauss(self,value):
        self.config['pw']['system']['degauss'] = value
    def nosym(self,value):
        self.config['pw']['system']['nosym'] = value
    def get_primitive(self,file='vasp-ase'):
        lattice,atoms,k_points = structure.primitive(self.poscar,file)
        self.config['pw']['atomic_positions'] = atoms
        self.config['pw']['cell_parameters']=lattice
    def soc(self,pseudo_path=False):
        if pseudo_path==False:
            pseudo_path = f"Research/{self.project_id}/REL"
        try:
            del self.config['pw']['system']['nspin']
        except:
            pass
        self.config['pw']['system']['noncolin']= True
        self.config['pw']['system']['lspinorb']= True
    def starting_potential(self,potential='file'):
        self.config['pw']['electrons']['startingpot']= potential
        if self.magnetism==True:
            self.config['pw']['system']['lforcet'] = True
    def magnetic_angle(self,angle1=[0,0],angle2=[0,0]):
        ntype = len(self.config['pw']['atomic_species']) #check number of atom types
        magnetic_atoms = [] #create magnetic atom index array
        for j,i in enumerate(self.config['pw']['atomic_species']): #for each atom
            if  i['atom']==self.magnetic_atom: # choose magnetic atom
                magnetic_atoms.append(j) #add atom magnetic atom list
            for k in range(ntype): #this for is necessary for AFM due to naming convention
                if  i['atom']==self.magnetic_atom+str(k): # choose magnetic atom
                    magnetic_atoms.append(j) #add atom magnetic atom list
        for j,i in enumerate(magnetic_atoms): #for all magnetic atoms
            self.config['pw']['system'][f'angle1({i+1})']=angle1[j] #define angle1 for atoms
            self.config['pw']['system'][f'angle2({j+1})']=angle2[j]#define angle2 for atoms
        
    def get_points(self,file_path=False,file_format=False):
        if file_format==False:
            file_format='qeinp-qetools'
        if file_path==False:
            file_path=f'./Projects/{self.project_id}/{self.job_id}/scf.in'
        try:
            k_points = structure.get_k(file_path,file_format)
        except:
            file_path=f'./Projects/{self.project_id}/{self.job_id}/vc-relax.in'
            k_points = structure.get_k(file_path,file_format)
        self.points=k_points
        return k_points
    
    def electron_maxstep(self,value):
        self.config['pw']['electrons']['electron_maxstep'] = value

    def exchange_maxstep(self,value):
        self.config['pw']['electrons']['exx_maxstep'] = value

    def mixing_beta(self,value):
        self.config['pw']['electrons']['mixing_beta'] = value

    def cell_dof(self,value):
        self.config['pw']['cell']['cell_dofree'] = value
        
    def get_structure(self,format,name=False,path=False,project_id=False,job_id=False,config=False):
        reads.read_structure(self,format,name=name,path=path)

    def calculate(self,calculation):
        self.set_calculation(calculation_type=calculation) #set calculation
        generate.input(self) #create input
        compute.run(self) #run calculation
        if calculation=='bands':
            self.set_calculation('bands-pp') #set calculation
            generate.input(self) #create input
            compute.run(self) #run calculation
        elif calculation=='pdos':
            utils.sumpdos(self)
        elif calculation=='kdos':
            utils.sumkdos(self)



    def check_convergence(self,calculation=False,job_id=False):
        if calculation==False:
            calculation=self.calculation
        if job_id == False:
            job_id = self.job_id
        path = f'./Projects/{self.project_id}/{job_id}/{calculation}.out'
        if calculation in ['relax','vc-relax']:
            try:
                isConverged = checks.check_relax(path=path)
            except:
                isConverged=False
        return isConverged



    def optimize(self,max_iter = 10,calculation=False):
        if calculation==False:
            calculation=self.calculation
        for i in range(max_iter):
            print(f'Starting {calculation} iteration {i+1} ')
            self.calculate(calculation)
            self.get_structure(calculation)
            isConverged = self.check_convergence(calculation)
            if isConverged:
                print(f'{calculation} is converged after {i+1} steps')
                break
        if i==max_iter-1 and not isConverged:
            print(f'{calculation} did not converged please increace number of steps steps')
                
    def dos(self,emin=False,emax=False,deltaE=False):
        if emin:
            self.config['dos']['dos']['emin']=emin
        if emax:
            self.config['dos']['dos']['emax']=emax
        if deltaE:
            self.config['dos']['dos']['deltaE']=deltaE
    def pdos(self,emin=False,emax=False,deltaE=False):
        if emin:
            self.config['projwfc']['projwfc']['emin']=emin
        if emax:
            self.config['projwfc']['projwfc']['emax']=emax
        if deltaE:
            self.config['projwfc']['projwfc']['deltaE']=deltaE
    def kdos(self,deltaE=0.005,ngauss=0,degauss=0.01):
            self.config['projwfc']['projwfc']['degauss']=degauss
            self.config['projwfc']['projwfc']['deltaE']=deltaE
            self.config['projwfc']['projwfc']['ngauss']=ngauss
    def test(self,parameter_name,start,end,step,conv_thr=False,num_core=1,debug=False,out=False,dual=4):
        result = utils.test_parameter(self=self,parameter_name=parameter_name,conv_thr=conv_thr,start=start,end=end,step=step,num_core=num_core,debug=debug,out=out,dual=dual)
        # if parameter=='ecutwfc':
        #     result = utils.test_ecutwfc(self=self,start=start,end=end,step=step,num_core=num_core,debug=debug)
        # elif parameter=='kpoints':
        #     result = utils.test_k(self=self,start=start,end=end,step=step,num_core=num_core,debug=debug)
        return result
    
    def set_q(self,nq1=2,nq2=2,nq3=2):
        self.config['ph']['inputph']['nq1']=nq1
        self.config['ph']['inputph']['nq2']=nq2
        self.config['ph']['inputph']['nq3']=nq3
    def set_path(self, path,number,poscar=True):
        self.get_structure(format='poscar')
        self.band_points(path,number)
        kpt = self.config['pw']['k_points_bands']
        self.config['pw']['k_points_bands']=kpt

    def export_structure(self,format='poscar',file_path=False,file_name=False,structure_name=False):
        atom = self.atoms() #Get atomic positions
        cell = self.cell()
        if  file_path == False:
            file_path = f'./Projects/{self.project_id}/{self.job_id}'
        if structure_name==False:
            if self.job_id=='results':
                structure_name = self.project_id
            else:
                structure_name = self.project_id+self.job_id
        if file_name==False:
            file_name=structure_name
        if format.lower()=='poscar':
            writes.write_poscar(structure_name=structure_name,atom=atom,cell=cell,file_name=file_name,file_path=file_path)



"""
This module contains the main functions to get a path and an explicit path.
"""
import numpy as np
import warnings
from . import SupercellWarning


def get_explicit_from_implicit(  # pylint: disable=too-many-locals
    seekpath_output, reference_distance
):
    """
    Given the output of ``get_path`` by seekpath, compute an "explicit" path,
    i.e. instead of just giving the endpoints and their coordinates, compute
    a full list of kpoints

    :param seekpath_output: a dictionary, the output of ``seekpath.get_path``

    :param reference_distance: a reference target distance between neighboring
        k-points in the path, in units of 1/ang. The actual value will be as
        close as possible to this value, to have an integer number of points in
        each path.
    """
    retdict = {}

    kpoints_rel = []
    kpoints_labels = []
    kpoints_linearcoord = []
    previous_linearcoord = 0.0
    segments = []
    for start_label, stop_label in seekpath_output["path"]:
        start_coord = np.array(seekpath_output["point_coords"][start_label])
        stop_coord = np.array(seekpath_output["point_coords"][stop_label])
        start_coord_abs = np.dot(
            start_coord, seekpath_output["reciprocal_primitive_lattice"]
        )
        stop_coord_abs = np.dot(
            stop_coord, seekpath_output["reciprocal_primitive_lattice"]
        )
        segment_length = np.linalg.norm(stop_coord_abs - start_coord_abs)
        num_points = max(2, int(segment_length / reference_distance))
        segment_linearcoord = np.linspace(0.0, segment_length, num_points)
        segment_start = len(kpoints_labels)
        for i in range(num_points):
            # Skip the first point if it's the same as the last one of
            # the previous segment
            if i == 0:
                if kpoints_labels:
                    if kpoints_labels[-1] == start_label:
                        segment_start -= 1
                        continue

            kpoints_rel.append(
                start_coord
                + (stop_coord - start_coord) * float(i) / float(num_points - 1)
            )
            if i == 0:
                kpoints_labels.append(start_label)
            elif i == num_points - 1:
                kpoints_labels.append(stop_label)
            else:
                kpoints_labels.append("")
            kpoints_linearcoord.append(previous_linearcoord + segment_linearcoord[i])
        previous_linearcoord += segment_length
        segment_end = len(kpoints_labels)
        segments.append((segment_start, segment_end))

    retdict["kpoints_rel"] = np.array(kpoints_rel)
    retdict["kpoints_linearcoord"] = np.array(kpoints_linearcoord)
    retdict["kpoints_labels"] = kpoints_labels
    retdict["kpoints_abs"] = np.dot(
        retdict["kpoints_rel"], seekpath_output["reciprocal_primitive_lattice"]
    )
    retdict["segments"] = segments

    return retdict


def get_path(
    structure,
    with_time_reversal=True,
    recipe="hpkot",
    threshold=1.0e-7,
    symprec=1e-05,
    angle_tolerance=-1.0,
):
    r"""
    Return the kpoint path information for band structure given a
    crystal structure, using the paths from the chosen recipe/reference.

    If you use this module, please cite the paper of the corresponding
    recipe (see parameter below).

    :param structure: The crystal structure for which we want to obtain
        the suggested path. It should be a tuple in the format
        accepted by spglib: ``(cell, positions, numbers)``, where
        (if N is the number of atoms):

        - ``cell`` is a :math:`3 \times 3` list of floats (``cell[0]`` is the first lattice
          vector, ...)
        - ``positions`` is a :math:`N \times 3` list of floats with the atomic coordinates
          in scaled coordinates (i.e., w.r.t. the cell vectors)
        - ``numbers`` is a length-:math:`N` list with integers identifying uniquely
          the atoms in the cell (e.g., the Z number of the atom, but
          any other positive non-zero integer will work - e.g. if you
          want to distinguish two Carbon atoms, you can set one number
          to 6 and the other to 1006)

    :param with_time_reversal: if False, and the group has no inversion
        symmetry, additional lines are returned as described in the HPKOT
        paper.

    :param recipe: choose the reference publication that defines the special
       points and paths.
       Currently, the following value is implemented:

       - ``hpkot``: HPKOT paper:
         Y. Hinuma, G. Pizzi, Y. Kumagai, F. Oba, I. Tanaka, Band structure
         diagram paths based on crystallography, Comp. Mat. Sci. 128, 140 (2017).
         DOI: 10.1016/j.commatsci.2016.10.015

    :param threshold: the threshold to use to verify if we are in
        and edge case (e.g., a tetragonal cell, but ``a==c``). For instance,
        in the tI lattice, if ``abs(a-c) < threshold``, a
        :py:exc:`~seekpath.hpkot.EdgeCaseWarning` is issued.
        Note that depending on the bravais lattice, the meaning of the
        threshold is different (angle, length, ...)

    :param symprec: the symmetry precision used internally by SPGLIB

    :param angle_tolerance: the angle_tolerance used internally by SPGLIB


    :return: a dictionary with the following
      keys:

        - ``point_coords``: a dictionary with label -> float coordinates
        - ``path``: a list of length-2 tuples, with the labels of the starting
          and ending point of each label section
        - ``has_inversion_symmetry``: True or False, depending on whether the
          input crystal structure has inversion symmetry or not.
        - ``augmented_path``: if True, it means that the path was
          augmented with the :math:`-k` points (this happens if both
          has_inversion_symmetry is False, and the user set
          with_time_reversal=False in the input)
        - ``bravais_lattice``: the Bravais lattice string (like ``cP``, ``tI``, ...)
        - ``bravais_lattice_extended``: the specific case used to define labels and
          coordinates (like ``cP1``, ``tI2``, ...)
        - ``cont_lattice``: three real-space vectors for the crystallographic
          conventional cell (``conv_lattice[0,:]`` is the first vector)
        - ``conv_positions``: fractional coordinates of atoms in the
          crystallographic conventional cell
        - ``conv_types``: list of integer types of the atoms in the crystallographic
          conventional cell (typically, the atomic numbers)
        - ``primitive_lattice``: three real-space vectors for the crystallographic
          primitive cell (``primitive_lattice[0,:]`` is the first vector)
        - ``primitive_positions``: fractional coordinates of atoms in the
          crystallographic primitive cell
        - ``primitive_types``: list of integer types of the atoms in the
          crystallographic primitive cell (typically, the atomic numbers)
        - ``reciprocal_primitive_lattice``: reciprocal-cell vectors for the
          primitive cell (vectors are rows: ``reciprocal_primitive_lattice[0,:]``
          is the first vector)
        - ``primitive_transformation_matrix``: the transformation matrix :math:`P` between
          the conventional and the primitive cell
        - ``inverse_primitive_transformation_matrix``: the inverse of the matrix :math:`P`
          (the determinant is integer and gives the ratio in volume between
          the conventional and primitive cells)
        - ``volume_original_wrt_conv``: volume ratio of the user-provided cell
          with respect to the the crystallographic conventional cell
        - ``volume_original_wrt_prim``: volume ratio of the user-provided cell
          with respect to the the crystalloraphic primitive cell

    :note: An :py:exc:`~seekpath.hpkot.EdgeCaseWarning` is issued for
        edge cases (e.g. if ``a==b==c`` for
        orthorhombic systems). In this case, still one of the valid cases
        is picked.
    """
    if recipe == "hpkot":
        from . import hpkot

        res = hpkot.get_path(
            structure=structure,
            with_time_reversal=with_time_reversal,
            threshold=threshold,
            symprec=symprec,
            angle_tolerance=angle_tolerance,
        )

    else:
        raise ValueError(
            "value for 'recipe' not recognized. The only value "
            "currently accepted is 'hpkot'."
        )
    return res


def get_explicit_k_path(
    structure,
    with_time_reversal=True,
    reference_distance=0.025,
    recipe="hpkot",
    threshold=1.0e-7,
    symprec=1e-05,
    angle_tolerance=-1.0,
):
    r"""
    Return the kpoint path for band structure (in scaled and absolute
    coordinates), given a crystal structure,
    using the paths proposed in the various publications (see description
    of the 'recipe' input parameter).
    Note that the k-path typically refers to a different unit cell
    (e.g., the primitive one with some transformation matrix to comply with
    the conventions in the case of the HPKOT paper). So, one should use the
    crystal cell provided in output for all calculations, rather than the
    input 'structure'.

    If you use this module, please cite the paper of the corresponding
    recipe (see parameter below).

    :param structure: The crystal structure for which we want to obtain
        the suggested path. It should be a tuple in the format
        accepted by spglib: (cell, positions, numbers), where
        (if N is the number of atoms):

        - ``cell`` is a :math:`3 \times 3` list of floats (``cell[0]`` is the first lattice
          vector, ...)
        - ``positions`` is a :math:`N \times 3` list of floats with the atomic coordinates
          in scaled coordinates (i.e., w.r.t. the cell vectors)
        - ``numbers`` is a length-:math:`N` list with integers identifying uniquely
          the atoms in the cell (e.g., the Z number of the atom, but
          any other positive non-zero integer will work - e.g. if you
          want to distinguish two Carbon atoms, you can set one number
          to 6 and the other to 1006)

    :param with_time_reversal: if False, and the group has no inversion
        symmetry, additional lines are returned.

    :param reference_distance: a reference target distance between neighboring
        k-points in the path, in units of 1/ang. The actual value will be as
        close as possible to this value, to have an integer number of points in
        each path.

    :param recipe: choose the reference publication that defines the special
       points and paths.
       Currently, the following value is implemented:

       - ``hpkot``: HPKOT paper:
         Y. Hinuma, G. Pizzi, Y. Kumagai, F. Oba, I. Tanaka, Band structure
         diagram paths based on crystallography, Comp. Mat. Sci. 128, 140 (2017).
         DOI: 10.1016/j.commatsci.2016.10.015

    :param threshold: the threshold to use to verify if we are in
        and edge case (e.g., a tetragonal cell, but ``a==c``). For instance,
        in the tI lattice, if ``abs(a-c) < threshold``, a
        :py:exc:`~seekpath.hpkot.EdgeCaseWarning` is issued.
        Note that depending on the bravais lattice, the meaning of the
        threshold is different (angle, length, ...)

    :param symprec: the symmetry precision used internally by SPGLIB

    :param angle_tolerance: the angle_tolerance used internally by SPGLIB

    .. versionchanged:: 1.8
        The key ``segments`` has been renamed ``explicit_segments``
        for consistency.

    :return: a dictionary with a number of keys. They are the same as
        ``get_path``, plus a few ones:

        - ``explicit_kpoints_abs``: List of the kpoints along the specific path in
          absolute (Cartesian) coordinates. The two endpoints are always
          included, independently of the length.
        - ``explicit_kpoints_rel``: List of the kpoints along the specific path in
          relative (fractional) coordinates (same length as
          explicit_kpoints_abs).
        - ``explicit_kpoints_labels``: list of strings with kpoints labels. It has
          the same length as explicit_kpoints_abs and explicit_kpoints_rel.
          Empty if the point is not a special point.
        - ``explicit_kpoints_linearcoord``: array of floats, giving the coordinate
          at which to plot the corresponding point.
        - ``explicit_segments``: a list of length-2 tuples, with the start and end
          index of each segment. **Note**! The indices are supposed to be
          used as follows: the labels for the i-th segment are given by::

            segment_indices = explicit_segments[i]
            segment_labels = explicit_kpoints_labels[slice(*segment_indices)]

          This means, in particular, that if you want the label of the start
          and end points, you should do::

            start_label = explicit_kpoints_labels[segment_indices[0]]
            stop_label = explicit_kpoints_labels[segment_indices[1]-1]

          (note the minus one!)

          Also, note that if
          ``explicit_segments[i-1][1] == explicit_segments[i][0] + 1`` it means
          that the point was calculated only once, and it belongs to both
          paths. Instead, if
          ``explicit_segments[i-1][1] == explicit_segments[i][0]``, then
          this is a 'break' point in the path (e.g., ``explicit_segments[i-1][1]``
          is the X point, and ``explicit_segments[i][0]`` is the R point,
          and typically in a graphical representation they are shown at the
          same coordinate, with a label ``R|X``).
    """
    if recipe == "hpkot":
        from . import hpkot

        res = hpkot.get_path(
            structure=structure,
            with_time_reversal=with_time_reversal,
            threshold=threshold,
            symprec=symprec,
            angle_tolerance=angle_tolerance,
        )

    else:
        raise ValueError(
            "value for 'recipe' not recognized. The only value "
            "currently accepted is 'hpkot'."
        )

    explicit_res = get_explicit_from_implicit(
        res, reference_distance=reference_distance
    )
    for k, v in explicit_res.items():
        res["explicit_{}".format(k)] = v
    return res


def get_path_orig_cell(
    structure,
    with_time_reversal=True,
    recipe="hpkot",
    threshold=1.0e-7,
    symprec=1e-05,
    angle_tolerance=-1.0,
):
    r"""
    Return the kpoint path information for band structure given a
    crystal structure, using the paths from the chosen recipe/reference.
    The original unit cell (i.e., the one provided in input by the user) is used.
    Standardization or symmetrization of the input unit cell is not performed.

    If the provided unit cell is a supercell of a smaller primitive cell,
    return the standard k path of the smaller primitive cell in the basis
    of the supercell reciprocal lattice vectors. In this case, the k-point
    labels lose their meaning as the corresponding k-points are not at the
    high-symmetry points of the first BZ of the given supercell. A
    :py:exc:`~seekpath.SupercellWarning` is issued.

    If you use this module, please cite the paper of the corresponding
    recipe (see parameter below).

    :param structure: The crystal structure for which we want to obtain
        the suggested path. It should be a tuple in the format
        accepted by spglib: ``(cell, positions, numbers)``, where
        (if N is the number of atoms):

        - ``cell`` is a :math:`3 \times 3` list of floats (``cell[0]`` is the first lattice
          vector, ...)
        - ``positions`` is a :math:`N \times 3` list of floats with the atomic coordinates
          in scaled coordinates (i.e., w.r.t. the cell vectors)
        - ``numbers`` is a length-:math:`N` list with integers identifying uniquely
          the atoms in the cell (e.g., the Z number of the atom, but
          any other positive non-zero integer will work - e.g. if you
          want to distinguish two Carbon atoms, you can set one number
          to 6 and the other to 1006)

    :param with_time_reversal: if False, and the group has no inversion
        symmetry, additional lines are returned as described in the HPKOT
        paper.

    :param recipe: choose the reference publication that defines the special
       points and paths.
       Currently, the following value is implemented:

       - ``hpkot``: HPKOT paper:
         Y. Hinuma, G. Pizzi, Y. Kumagai, F. Oba, I. Tanaka, Band structure
         diagram paths based on crystallography, Comp. Mat. Sci. 128, 140 (2017).
         DOI: 10.1016/j.commatsci.2016.10.015

    :param threshold: the threshold to use to verify if we are in
        and edge case (e.g., a tetragonal cell, but ``a==c``). For instance,
        in the tI lattice, if ``abs(a-c) < threshold``, a
        :py:exc:`~seekpath.hpkot.EdgeCaseWarning` is issued.
        Note that depending on the bravais lattice, the meaning of the
        threshold is different (angle, length, ...)

    :param symprec: the symmetry precision used internally by SPGLIB

    :param angle_tolerance: the angle_tolerance used internally by SPGLIB


    :return: a dictionary with the following
      keys:

        - ``point_coords``: a dictionary with label -> float coordinates
        - ``path``: a list of length-2 tuples, with the labels of the starting
          and ending point of each label section
          input crystal structure has inversion symmetry or not.
        - ``augmented_path``: if True, it means that the path was
          augmented with the :math:`-k` points (this happens if both
          has_inversion_symmetry is False, and the user set
          with_time_reversal=False in the input)
        - ``is_supercell``: True if the input unit cell is a supercell of
          a smaller primitive cell.

    :note: An :py:exc:`~seekpath.hpkot.EdgeCaseWarning` is issued for
        edge cases (e.g. if ``a==b==c`` for
        orthorhombic systems). In this case, still one of the valid cases
        is picked.
    """

    res = get_path(
        structure=structure,
        with_time_reversal=with_time_reversal,
        threshold=threshold,
        symprec=symprec,
        angle_tolerance=angle_tolerance,
        recipe=recipe,
    )

    is_supercell = abs(res["volume_original_wrt_prim"] - 1) > 0.1

    if is_supercell:
        warnings.warn(
            "The provided cell is a supercell: the returned k-path is the "
            "standard k-path of the associated primitive cell in the basis of "
            "the supercell reciprocal lattice.",
            SupercellWarning,
        )

    # points in the output of get_path are in scaled coordinates of the
    # standardized primitive lattice
    points_scaled_standard = res["point_coords"]

    # Convert points from scaled coordinates of the standardiced primitive
    # lattice to Cartesian coordinates
    points_cartesian = {}
    for pointname, coords in points_scaled_standard.items():
        points_cartesian[pointname] = coords @ np.array(
            res["reciprocal_primitive_lattice"]
        )

    # Rotate points in Cartesian space
    for pointname, coords in points_cartesian.items():
        points_cartesian[pointname] = coords @ res["rotation_matrix"]

    # Convert points from Cartesian coordinates to the scaled coordinates
    # of the original lattice
    points_scaled_original = {}
    cell_orig = np.array(structure[0])
    for pointname, coords in points_cartesian.items():
        points_scaled_original[pointname] = list(coords @ cell_orig.T / np.pi / 2)

    res_orig = {
        "point_coords": points_scaled_original,
        "path": res["path"],
        "augmented_path": res["augmented_path"],
        "is_supercell": is_supercell,
        "has_inversion_symmetry": res["has_inversion_symmetry"],
        "bravais_lattice": res["bravais_lattice"],
        "bravais_lattice_extended": res["bravais_lattice_extended"],
        "spacegroup_number": res["spacegroup_number"],
        "spacegroup_international": res["spacegroup_international"],
    }

    return res_orig


def get_explicit_k_path_orig_cell(
    structure,
    with_time_reversal=True,
    reference_distance=0.025,
    recipe="hpkot",
    threshold=1.0e-7,
    symprec=1e-05,
    angle_tolerance=-1.0,
):
    r"""
    Return the kpoint path for band structure (in scaled and absolute
    coordinates), given a crystal structure,
    using the paths proposed in the various publications (see description
    of the 'recipe' input parameter) for the given unit cell.
    Standardization or symmetrization of the input unit cell is not performed.

    If the provided unit cell is a supercell of a smaller primitive cell,
    return the standard k path of the smaller primitive cell in the basis
    of the supercell reciprocal lattice vectors. In this case, the k-point
    labels lose their meaning as the corresponding k-points are not at the
    high-symmetry points of the first BZ of the given supercell. A
    :py:exc:`~seekpath.SupercellWarning` is issued.

    If you use this module, please cite the paper of the corresponding
    recipe (see parameter below).

    :param structure: The crystal structure for which we want to obtain
        the suggested path. It should be a tuple in the format
        accepted by spglib: (cell, positions, numbers), where
        (if N is the number of atoms):

        - ``cell`` is a :math:`3 \times 3` list of floats (``cell[0]`` is the first lattice
          vector, ...)
        - ``positions`` is a :math:`N \times 3` list of floats with the atomic coordinates
          in scaled coordinates (i.e., w.r.t. the cell vectors)
        - ``numbers`` is a length-:math:`N` list with integers identifying uniquely
          the atoms in the cell (e.g., the Z number of the atom, but
          any other positive non-zero integer will work - e.g. if you
          want to distinguish two Carbon atoms, you can set one number
          to 6 and the other to 1006)

    :param with_time_reversal: if False, and the group has no inversion
        symmetry, additional lines are returned.

    :param reference_distance: a reference target distance between neighboring
        k-points in the path, in units of 1/ang. The actual value will be as
        close as possible to this value, to have an integer number of points in
        each path.

    :param recipe: choose the reference publication that defines the special
       points and paths.
       Currently, the following value is implemented:

       - ``hpkot``: HPKOT paper:
         Y. Hinuma, G. Pizzi, Y. Kumagai, F. Oba, I. Tanaka, Band structure
         diagram paths based on crystallography, Comp. Mat. Sci. 128, 140 (2017).
         DOI: 10.1016/j.commatsci.2016.10.015

    :param threshold: the threshold to use to verify if we are in
        and edge case (e.g., a tetragonal cell, but ``a==c``). For instance,
        in the tI lattice, if ``abs(a-c) < threshold``, a
        :py:exc:`~seekpath.hpkot.EdgeCaseWarning` is issued.
        Note that depending on the bravais lattice, the meaning of the
        threshold is different (angle, length, ...)

    :param symprec: the symmetry precision used internally by SPGLIB

    :param angle_tolerance: the angle_tolerance used internally by SPGLIB

    .. versionchanged:: 1.8
        The key ``segments`` has been renamed ``explicit_segments``
        for consistency.

    :return: a dictionary with a number of keys. They are the same as
        ``get_path_orig_cell``, plus a few ones:

        - ``explicit_kpoints_abs``: List of the kpoints along the specific path in
          absolute (Cartesian) coordinates. The two endpoints are always
          included, independently of the length.
        - ``explicit_kpoints_rel``: List of the kpoints along the specific path in
          relative (fractional) coordinates (same length as
          explicit_kpoints_abs).
        - ``explicit_kpoints_labels``: list of strings with kpoints labels. It has
          the same length as explicit_kpoints_abs and explicit_kpoints_rel.
          Empty if the point is not a special point.
        - ``explicit_kpoints_linearcoord``: array of floats, giving the coordinate
          at which to plot the corresponding point.
        - ``explicit_segments``: a list of length-2 tuples, with the start and end
          index of each segment. **Note**! The indices are supposed to be
          used as follows: the labels for the i-th segment are given by::

            segment_indices = explicit_segments[i]
            segment_labels = explicit_kpoints_labels[slice(*segment_indices)]

          This means, in particular, that if you want the label of the start
          and end points, you should do::

            start_label = explicit_kpoints_labels[segment_indices[0]]
            stop_label = explicit_kpoints_labels[segment_indices[1]-1]

          (note the minus one!)

          Also, note that if
          ``explicit_segments[i-1][1] == explicit_segments[i][0] + 1`` it means
          that the point was calculated only once, and it belongs to both
          paths. Instead, if
          ``explicit_segments[i-1][1] == explicit_segments[i][0]``, then
          this is a 'break' point in the path (e.g., ``explicit_segments[i-1][1]``
          is the X point, and ``explicit_segments[i][0]`` is the R point,
          and typically in a graphical representation they are shown at the
          same coordinate, with a label ``R|X``).
    """
    from .hpkot.tools import get_reciprocal_cell_rows

    res = get_path_orig_cell(
        structure=structure,
        with_time_reversal=with_time_reversal,
        threshold=threshold,
        symprec=symprec,
        angle_tolerance=angle_tolerance,
        recipe=recipe,
    )

    # Set reciprocal_primitive_lattice as the reciprocal lattice of the original
    # cell. To be used only in the get_explicit_from_implicit function.
    res["reciprocal_primitive_lattice"] = get_reciprocal_cell_rows(structure[0])

    explicit_res = get_explicit_from_implicit(
        res, reference_distance=reference_distance
    )

    res.pop("reciprocal_primitive_lattice")

    for k, v in explicit_res.items():
        res["explicit_{}".format(k)] = v
    return res



import subprocess

def run(self):
    if self.debug == False:
        print(f'{self.calculation} for {self.job_id} is started')
        if self.gpu==True:
                p = subprocess.Popen(f"srun --mpi=pmi2 singularity run --nv  /work/bansil/s.sevim/software/qe-gpu/qe-7.1-gpu.sif {self.package}.x -npool {self.num_gpu} -inp ./Projects/{self.project_id}/{self.job_id}/{self.calculation}.in > ./Projects/{self.project_id}/{self.job_id}/{self.calculation}.out ", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
                p.wait()
        else:
                p = subprocess.Popen(f"mpirun -np {self.num_core} {self.package}.x -inp ./Projects/{self.project_id}/{self.job_id}/{self.calculation}.in > ./Projects/{self.project_id}/{self.job_id}/{self.calculation}.out ", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
                p.wait()
        print(f'{self.calculation} for {self.job_id} is finished')

def sumpdos(self,atom,orbital):
        p = subprocess.Popen(f"sumpdos.x ./Projects/{self.project_id}/{self.job_id}/*\({atom}\)*\({orbital}\) > ./Projects/{self.project_id}/{self.job_id}/sumpdos_{atom}_{orbital}.dat", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        p.wait()

def sumkdos(self,atom,orbital):
        p = subprocess.Popen(f"sumpdos.x ./Projects/{self.project_id}/{self.job_id}/dos.k*\({atom}\)*\({orbital}\) > ./Projects/{self.project_id}/{self.job_id}/sumkdos_{atom}_{orbital}.dat", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        p.wait()


import sys
import argparse
import json
from src import generate

# Construct the argument parser
ap = argparse.ArgumentParser()

# Add the arguments to the parser
ap.add_argument("-n", "--name", required=True,
                help="Project name")
ap.add_argument("-c", "--calculation", required=True,
                help="Type of calculation")
ap.add_argument("-j", "--json",  required=True,
                help="JSON file")
ap.add_argument("-d", "--degauss", required=False,
                help="Degauss value")
ap.add_argument("-k", "--kpoints", nargs='+', required=False,
                help="K points")
ap.add_argument("-i", "--initial",  required=False,
                help="Initial guess path")
ap.add_argument("-p", "--poscar",  required=False,
                help="Poscar path")
ap.add_argument("-l", "--layer",  required=False,
                help="Layered materials")



args = vars(ap.parse_args())

project_id = args['name']
# Type of calculation
calculation = args['calculation']
# Degauss parameter
degauss = args['degauss']
parameter = args['json']
# K points

if args['kpoints'] != None:
    k_points = ' '.join(args['kpoints'])
else:
    k_points = None

layer = args["layer"]


initial = args['initial']
poscar = args['poscar']
# print(f"Initial path: {initial}")
# print(f"Initial path: {poscar}")

with open(f'{parameter}') as f:
        data = f.read()
        config = json.loads(data)


generate.pw_input(project_id=project_id,calculation=calculation,config=config,degauss=degauss, k_points=k_points,
         initial_guess=initial, poscar=poscar,layer=layer)



import numpy as np
def write_atom_species(file,atomic_species):
    with open(file, "a") as file_object:
        file_object.write("ATOMIC_SPECIES \n")
        for atom in atomic_species:
            listed = list(atom.values())
            file_object.write(" ".join(listed)+'\n')


def write_atom_positions(file, positions):
    with open(file, "a") as file_object:
        file_object.write("ATOMIC_POSITIONS (crystal) \n")
        for i in positions:
            try:
                file_object.write(" ".join(i)+'\n')
            except:
                try:
                    file_object.write(" ".join(i.astype(str))+'\n')
                except:
                    file_object.write(" ".join(str(i))+'\n')

def write_cell_parameters(file, cell):
    try:
        cell = np.array(cell,float)
    except:
        print('Something is wrong with the cell parameters. The error got caught on input file writing step.')
    with open(file, "a") as file_object:
        file_object.write("CELL_PARAMETERS (angstrom) \n")
        for i in cell:
            for k,j in enumerate(i):
                file_object.write(f"{np.around(j,8):<.10f}  ")
            file_object.write("\n")


def write_k_points(file, k):
    with open(file, "a") as file_object:
        file_object.write("K_POINTS automatic \n")
        file_object.write(k+'\n')

def write_k_points_bands(file,k):
    with open(file, "a") as file_object:
        file_object.write("K_POINTS crystal_b \n")
        file_object.write(str(len(k))+'\n')
        for point in k:
            listed = list(point.values())
            file_object.write(" ".join(listed)+'\n')

def write_k_points_matdyn(file,k):
    with open(file, "a") as file_object:
        file_object.write(str(len(k))+'\n')
        for point in k:
            listed = list(point.values())
            file_object.write(" ".join(listed)+'\n')


def write_hubbard(file,parameter):
    with open(file, "a") as file_object:
        file_object.write(f"HUBBARD {parameter['projection']} \n")
        for term in parameter['terms']:
            file_object.write(f"{term['interaction']} {term['atom']}-{term['orbital']} {term['value']}\n")


def write_poscar(structure_name,atom,cell,file_name='POSCAR',file_path='./'):
    full_path = file_path+"/"+file_name+".poscar"
    atom_name  = np.array(atom).T[0].tolist()
    atom_type = {} 
    for i in atom_name:
        if i in atom_type:
            atom_type[i]+=1
        else:
            atom_type[i]=1
    with open(full_path, "w") as file_object:
        file_object.write(f"{structure_name} \n")
        file_object.write("1.0 \n")
        for i in cell:
            for k,j in enumerate(i):
                file_object.write(f"{np.around(float(j),10):<.12f}  ")
            file_object.write("\n")
        for i in atom_type:
            file_object.write(f"{i}  ")
        file_object.write("\n")
        for i in atom_type:
            file_object.write(f"{atom_type[i]}  ")
        file_object.write("\n")
        file_object.write("direct")
        file_object.write("\n")
        for i in atom:
            for k,j in enumerate(i[1:]):
                file_object.write(f"{np.around(float(j),8):<.10f}  ")
            file_object.write(f"{i[0]}  ")
            file_object.write("\n")


import numpy as np
import json
from . import utils

def read_structure(self,format,path=False,name=False):
    config = self.config['pw']
    project_id = self.project_id
    job_id = self.job_id
    if format.lower()=='poscar':
        if not name:
            name=project_id
        if not path:
            path='./'
        cell,atom = read_poscar(f"{path}/{name}")  
    if format.lower()=='vc-relax':
        cell, atom = read_vc_relax(f"./Projects/{project_id}/{job_id}/vc-relax.out")
    if format.lower()=='relax':
        atom = read_relax(f"./Projects/{project_id}/{job_id}/relax.out")
    try:
        config['cell_parameters'], config['atomic_positions'] = cell,atom
    except:
        try:
            config['atomic_positions'] = atom
        except:
            return cell,atom
    config['atomic_species']=utils.default_pseudo(atom)
    ntyp = len(config['atomic_species'])
    config['system']['ntyp']=ntyp


    try:
        if self.magnetic_order=='afm':
            #Check magnetic atoms order 
            for k,i in enumerate(config['atomic_species']):
                for j in i['atom']:
                    try:
                        del config['system'][f'starting_magnetization({k+1})']
                    except:
                        pass
                    # print(j)
                    try:
                        int(j)
                        if int(j)==0:
                            config['system'][f'starting_magnetization({k+1})']=-1
                        elif int(j)==1:
                            config['system'][f'starting_magnetization({k+1})']=1

                    except:
                        pass
    except:
        pass
    try:
        if self.magnetic_order=='fm':
            #Check magnetic atoms order 
            for k,i in enumerate(config['atomic_species']):
                for j in i['atom']:
                    try:
                        del config['system'][f'starting_magnetization({k+1})']
                    except:
                        pass
            for j,i in enumerate(self.config['pw']['atomic_species']): #for each atom
                if  i['atom']==self.magnetic_atom: # choose magnetic atom
                    self.config['pw']['system'][f'starting_magnetization({j+1})']=1
                    if (self.angle1):
                        self.config['pw']['system'][f'angle1({j+1})']=self.angle1
                    if (self.angle2):
                        self.config['pw']['system'][f'angle2({j+1})']=self.angle2
    except:
        pass


    # print(cell,atom)
        
def read_pp(paths):
    elements = []
    wavefunction = []
    chargedensity = []
    for path in paths:
        pp = {"Element":"","Wavefunction":50,"Chargedensity:":200}
        with open(path, 'r') as data:
            data = data.read().split()
            for i in range(200):
                if data[i] =="Element:":
                    # print(str(f"Element: {data[i+1]}"))
                    pp["Element"]=data[i+1]
                if data[i] =="wavefunctions:":
                    # print(f"Wave function cutoff: {float(data[i+1])}")
                    pp["Wavefunction"]=data[i+1]
                if data[i] =="density:":
                    # print(f"Charge density cutoff: {float(data[i+1])}")
                    pp["Chargedensity"]=data[i+1]

        elements.append(pp['Element'])
        wavefunction.append(float(pp['Wavefunction']))
        chargedensity.append(float(pp['Chargedensity']))
    chargedensity.sort()
    wavefunction.sort()
    return(elements,wavefunction[-1]*1.5,chargedensity[-1]*1.5)


def read_poscar(path):
    data = open(path, 'r').readlines()
    cell =[]
    for i in data[2:5]:
        line=i.split()
        cell.append(line)
    atoms= []
    for i in data[8:]:
        line=(i.split())
        position=line[:3] 
        atom = line[-1]
        for j in atom:
            try:
                int(j)
                atom = atom[:-2]
            except:
                pass
        atoms.append([atom]+position)
    return(cell,atoms)
    
        
def read_vc_relax(path):
    vc_relax = open(path, 'r').readlines()
    begin = 0
    end = 0
    for i in range(len(vc_relax)):
        if vc_relax[i] == 'Begin final coordinates\n':
            begin = i
        if vc_relax[i] == 'End final coordinates\n':
            end = i
    cell = np.array([i.split()
                    for i in vc_relax[begin+5:begin+8]]).astype(float)
    atoms = np.array([i.split() for i in vc_relax[begin+10:end]])
    return(cell, atoms)

def read_relax(path):
    data = open(path, 'r').readlines()
    begin = 0
    end = 0
    for i in range(len(data)):
        if data[i] == 'Begin final coordinates\n':
            begin = i
        if data[i] == 'End final coordinates\n':
            end = i
    atoms = np.array([i.split() for i in data[begin+3:end]])
    return atoms

def read_symmetries(fstring):
    f = open(fstring, 'r')
    x = np.zeros(0)
    for i in f:
        if "high-symmetry" in i:
            x = np.append(x, float(i.split()[-1]))
    f.close()
    return x

def read_efermi(path):
    lines = open(path, 'r').readlines()
    e_fermi = 0
    for i in lines:
        if "the Fermi energy is" in i:
            e_fermi = float(i.split()[-2])
            return e_fermi

def read_num_bands(self):
    directory = f"./Projects/{self.project_id}/{self.job_id}/scf.out"
    lines = open(directory, 'r').readlines()
    number = 0
    for i in lines:
        if "    number of Kohn-Sham states" in i:
            number = float(i.split()[-1])
            return number
        

def read_json(data=False,path=False):
    if (path):
        with open(path) as f:
            data = f.read()
    crystal = json.loads(data)
    atoms = []
    cell = np.array(crystal['lattice']['matrix'],dtype=str)
    for i in range(len(crystal['sites'])):
        element = crystal['sites'][i]['species'][0]['element']
        coordinate = crystal['sites'][i]['abc']
        atoms.append([element,str(coordinate[0]),str(coordinate[1]),str(coordinate[2])])
        # atoms.append(element)
        # coordinate = crystal['sites'][i]['abc']
        # positions[i]={element:coordinate}
    return cell,atoms
 



from . import seekpath
from markupsafe import escape
import copy
import io
import json
import time
import traceback

import numpy as np

from ase.data import chemical_symbols
from tools_barebone import logme, get_tools_barebone_version
from tools_barebone.structure_importers import get_structure_tuple, UnknownFormatError




MAX_NUMBER_OF_ATOMS = 1000


def get_json_for_visualizer(
    cell, relcoords, atomic_numbers
):  # pylint: disable=too-many-locals
    from seekpath import hpkot, brillouinzone
    hpkot = seekpath.hpkot

    system = (np.array(cell), np.array(relcoords), np.array(atomic_numbers))
    res = hpkot.get_path(system, with_time_reversal=False)

    real_lattice = res["primitive_lattice"]
    # rec_lattice = np.linalg.inv(real_lattice).T # Missing 2pi!
    rec_lattice = np.array(hpkot.tools.get_reciprocal_cell_rows(real_lattice))
    b1, b2, b3 = rec_lattice
    faces_data = seekpath.brillouinzone.get_BZ(b1=b1, b2=b2, b3=b3)

    response = {}
    response["faces_data"] = faces_data
    response["b1"] = b1.tolist()
    response["b2"] = b2.tolist()
    response["b3"] = b3.tolist()
    ## Convert to absolute
    response["kpoints"] = {
        k: (v[0] * b1 + v[1] * b2 + v[2] * b3).tolist()
        for k, v in res["point_coords"].items()
    }
    response["kpoints_rel"] = {
        k: [v[0], v[1], v[2]] for k, v in res["point_coords"].items()
    }
    response["path"] = res["path"]

    # It should use the same logic, so give the same cell as above
    res_explicit = seekpath.get_explicit_k_path(system, with_time_reversal=False)
    for k in res_explicit:
        if k == "segments" or k.startswith("explicit_"):
            if isinstance(res_explicit[k], np.ndarray):
                response[k] = res_explicit[k].tolist()
            else:
                response[k] = res_explicit[k]

    if (
        np.sum(
            np.abs(
                np.array(res_explicit["reciprocal_primitive_lattice"])
                - np.array(res["reciprocal_primitive_lattice"])
            )
        )
        > 1.0e-7
    ):
        raise AssertionError("Got different reciprocal cells...")

    # Response for JS, and path_results
    return response, res


def process_structure_core(  # pylint: disable=too-many-locals,too-many-statements,too-many-arguments
    filecontent,
    fileformat,
    call_source="",
    logger=None,
    flask_request=None,
):
    
    start_time = time.time()
    fileobject = io.StringIO(str(filecontent))
    # form_data = dict(flask_request.form)
    form_data = None

    structure_tuple = get_structure_tuple(
            fileobject, fileformat, extra_data=None
        )
    if len(structure_tuple[1]) > MAX_NUMBER_OF_ATOMS:
        ## Structure too big
        logme(
            logger,
            filecontent,
            fileformat,
            flask_request,
            call_source,
            reason="toolarge",
            extra={"number_of_atoms": len(structure_tuple[1]), "form_data": form_data,},
        )
        raise FlaskRedirectException(
            "Sorry, this online visualizer is limited to {} atoms "
            "in the input cell, while your structure has {} atoms."
            "".format(MAX_NUMBER_OF_ATOMS, len(structure_tuple[1]))
        )

    # Log the content in case of valid structure
    logme(
        logger,
        filecontent,
        fileformat,
        flask_request,
        call_source,
        reason="OK",
        extra={"number_of_atoms": len(structure_tuple[1]), "form_data": form_data,},
    )

    try:
        in_json_data = {
            "cell": structure_tuple[0],
            "scaled_coords": structure_tuple[1],
            "atomic_numbers": structure_tuple[2],
        }

        out_json_data, path_results = get_json_for_visualizer(
            in_json_data["cell"],
            in_json_data["scaled_coords"],
            in_json_data["atomic_numbers"],
        )

        raw_code_dict = copy.copy(out_json_data)
        for k in list(raw_code_dict.keys()):
            if k.startswith("explicit_"):
                raw_code_dict.pop(k)
            if k == "segments":
                raw_code_dict.pop(k)
        raw_code_dict.pop("faces_data")
        raw_code_dict["primitive_lattice"] = path_results["primitive_lattice"].tolist()
        raw_code_dict["primitive_positions"] = path_results[
            "primitive_positions"
        ].tolist()
        inputstructure_positions_cartesian = np.dot(
            np.array(in_json_data["scaled_coords"]), np.array(in_json_data["cell"]),
        ).tolist()
        primitive_positions_cartesian = np.dot(
            np.array(path_results["primitive_positions"]),
            np.array(path_results["primitive_lattice"]),
        ).tolist()
        primitive_positions_cartesian_refolded = np.dot(
            np.array(path_results["primitive_positions"]) % 1.0,
            np.array(path_results["primitive_lattice"]),
        ).tolist()
        raw_code_dict["primitive_positions_cartesian"] = primitive_positions_cartesian

        # raw_code['primitive_types'] = path_results['primitive_types']
        primitive_symbols = [
            chemical_symbols[num] for num in path_results["primitive_types"]
        ]
        raw_code_dict["primitive_symbols"] = primitive_symbols

        raw_code = json.dumps(raw_code_dict, indent=2)
        ## I manually escape it to then add <br> and pass it to a filter with
        ## |safe. I have to 'unicode' it otherwise it keeps escaping also the
        ## next replaces
        raw_code = (
            str(escape(raw_code)).replace("\n", "<br>").replace(" ", "&nbsp;")
        )

        kpoints = [
            [
                k,
                out_json_data["kpoints"][k][0],
                out_json_data["kpoints"][k][1],
                out_json_data["kpoints"][k][2],
            ]
            for k in sorted(out_json_data["kpoints"])
        ]
        kpoints_rel = [
            [
                k,
                out_json_data["kpoints_rel"][k][0],
                out_json_data["kpoints_rel"][k][1],
                out_json_data["kpoints_rel"][k][2],
            ]
            for k in sorted(out_json_data["kpoints_rel"])
        ]

        inputstructure_cell_vectors = [
            [idx, coords[0], coords[1], coords[2]]
            for idx, coords in enumerate(in_json_data["cell"], start=1)
        ]
        inputstructure_symbols = [
            chemical_symbols[num] for num in in_json_data["atomic_numbers"]
        ]
        inputstructure_atoms_scaled = [
            [label, coords[0], coords[1], coords[2]]
            for label, coords in zip(
                inputstructure_symbols, in_json_data["scaled_coords"]
            )
        ]
        inputstructure_atoms_cartesian = [
            [label, coords[0], coords[1], coords[2]]
            for label, coords in zip(
                inputstructure_symbols, inputstructure_positions_cartesian
            )
        ]

        direct_vectors = [
            [idx, coords[0], coords[1], coords[2]]
            for idx, coords in enumerate(path_results["primitive_lattice"], start=1)
        ]

        reciprocal_primitive_vectors = [
            [idx, coords[0], coords[1], coords[2]]
            for idx, coords in enumerate(
                path_results["reciprocal_primitive_lattice"], start=1
            )
        ]

        atoms_scaled = [
            [label, str(coords[0]), str(coords[1]), str(coords[2])]
            for label, coords in zip(
                primitive_symbols, path_results["primitive_positions"]
            )
        ]

        atoms_cartesian = [
            [label, coords[0], coords[1], coords[2]]
            for label, coords in zip(primitive_symbols, primitive_positions_cartesian)
        ]

        # Create extetically-nice looking path, with dashes and pipes
        suggested_path = []
        if path_results["path"]:
            suggested_path.append(path_results["path"][0][0])
            suggested_path.append("-")
            suggested_path.append(path_results["path"][0][1])
            last = path_results["path"][0][1]
        for p1, p2 in path_results["path"][1:]:
            if p1 != last:
                suggested_path.append("|")
                suggested_path.append(p1)
            suggested_path.append("-")
            suggested_path.append(p2)
            last = p2

        primitive_lattice = path_results["primitive_lattice"]
        xsfstructure = []
        xsfstructure.append("CRYSTAL")
        xsfstructure.append("PRIMVEC")
        for vector in primitive_lattice:
            xsfstructure.append("{} {} {}".format(vector[0], vector[1], vector[2]))
        xsfstructure.append("PRIMCOORD")
        xsfstructure.append("{} 1".format(len(primitive_positions_cartesian_refolded)))
        for atom_num, pos in zip(
            path_results["primitive_types"], primitive_positions_cartesian_refolded
        ):
            xsfstructure.append("{} {} {} {}".format(atom_num, pos[0], pos[1], pos[2]))
        xsfstructure = "\n".join(xsfstructure)

        compute_time = time.time() - start_time
    except Exception:
        logme(
            logger,
            filecontent,
            fileformat,
            flask_request,
            call_source,
            reason="codeexception",
            extra={"traceback": traceback.extract_stack(), "form_data": form_data,},
        )
        raise
    # print(raw_code_dict)
    # print(direct_vectors)
    return dict(
        jsondata=json.dumps(out_json_data),
        volume_ratio_prim=int(round(path_results["volume_original_wrt_prim"])),
        raw_code=raw_code,
        kpoints=kpoints,
        kpoints_rel=kpoints_rel,
        bravais_lattice=path_results["bravais_lattice"],
        bravais_lattice_extended=path_results["bravais_lattice_extended"],
        spacegroup_number=path_results["spacegroup_number"],
        spacegroup_international=path_results["spacegroup_international"],
        direct_vectors=direct_vectors,
        inputstructure_cell_vectors=inputstructure_cell_vectors,
        inputstructure_atoms_scaled=inputstructure_atoms_scaled,
        inputstructure_atoms_cartesian=inputstructure_atoms_cartesian,
        atoms_scaled=atoms_scaled,
        with_without_time_reversal=(
            "with" if path_results["has_inversion_symmetry"] else "without"
        ),
        atoms_cartesian=atoms_cartesian,
        reciprocal_primitive_vectors=reciprocal_primitive_vectors,
        primitive_vectors=primitive_lattice,
        suggested_path=suggested_path,
      )


def primitive(fileContent,fileformat):
    with open(fileContent) as file:
        fileContent = file.read()
    process_structure_core(filecontent=fileContent,fileformat=fileformat)
    structure = process_structure_core(filecontent=fileContent,fileformat=fileformat)
    lattice = (structure["primitive_vectors"].astype('str')).tolist()
    atoms = structure["atoms_scaled"]
    kpoints = structure['kpoints_rel']
    return (lattice,atoms,kpoints)


def get_k(fileContent,fileformat):
    with open(fileContent) as file:
        fileContent = file.read()
    structure = process_structure_core(filecontent=fileContent,fileformat=fileformat)
    kpoints = structure['kpoints_rel']
    return (kpoints)


import paramiko
from dotenv import dotenv_values

config = dotenv_values(".env")
hostname=config['hostname']
username=config['username']
password=config['password']

def exec(command):
    with paramiko.SSHClient() as client:
        client.load_system_host_keys()
        client.set_missing_host_key_policy(paramiko.WarningPolicy)

        # Establish SSH connection
        client.connect(hostname, username=username)

        # Submit our Grid Engine job by running a remote 'qsub' command over SSH
        stdin, stdout, stderr = client.exec_command(command)

        # Show the standard output and error of our job
        print("Standard output:")
        out = stdout.read()
        lines = out.decode('utf-8').split("\n")
        for line in lines:
            print(line)

def put(payload,destination):
    try:
        hostname=config['sftp']
    except:
        pass
    with paramiko.SSHClient() as client:
        client.load_system_host_keys()
        # client.set_missing_host_key_policy(paramiko.WarningPolicy)

        # Establish SSH connection
        client.connect(hostname, username=username,password=password)


        # # Establish SFTP connection
        with client.open_sftp() as sftp:
        #     # Push job submission script to a particular path on the cluster
            sftp.put(payload, destination)


def get(payload,destination):
    try:
        hostname=config['sftp']
    except:
        pass
    with paramiko.SSHClient() as client:
        client.load_system_host_keys()
        # client.set_missing_host_key_policy(paramiko.WarningPolicy)

        # Establish SSH connection
        client.connect(hostname, username=username,password=password)


        # # Establish SFTP connection
        with client.open_sftp() as sftp:
        #     # Push job submission script to a particular path on the cluster
            sftp.get(payload, destination)




"""
Sample setup.py file
"""
from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="esma",
    version='0.0.229',
    author="Susy Exists",
    author_email="susy@selectron.me",
    description = "Quantum Espresso automation tool",
    url = "https://github.com/susyexstsi/espresso-machine",
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=[  "numpy",
  "matplotlib",
  "pandas",
  "qcelemental",
  "ase",
  "Flask",
  "qe-tools",
  "scipy",
  "seekpath",
  "untangle",
  "tools_barebone",
  "ipykernel",
  "ipython"],
    keywords=['pypi', 'cicd', 'python'],
    classifiers=[
"Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",]
)


from . import utils
from . import reads
from . import generate
from . import compute
from . import kpoints
from . import plots
from . import structure
from .project import project


class ph:
    def __init__(self,project_id):
        self.project_id = project_id
        self.config = utils.configure('ph')
        self.job_id = 'results'
        self.calculation = False
    def create_input(self,job_id='results'):
        # self.set_calculation(self.calculation)
        generate.ph_input(project_id=self.project_id,calculation=self.calculation,job_id=self.job_id, config=self.config)
    def set_calculation(self,calculation):
        self.calculation=calculation
        self.config = utils.configure(calculation)
    def calculate(self, num_core=1):
        compute.run_ph(self.project_id,self.job_id,self.calculation,num_core)
    def set_q(self,nq1=2,nq2=2,nq3=2):
        self.config['inputph']['nq1']=nq1
        self.config['inputph']['nq2']=nq2
        self.config['inputph']['nq3']=nq3
    def set_path(self, path,number,poscar=True):
        model = pw(project_id=self.project_id )
        if (poscar):
            model.get_structure(format='poscar')
        model.band_points(path,number)
        kpt = model.config['k_points_bands']
        self.config['k_points_bands']=kpt
    def plot(self):
        plots.plot_phonon(self)





point_coordinate = {'K':[0.33333333333,0.3333333333,0],'M':[0,0.5,0],'G':[0,0,0]}

def band_points (points):
    point_list =[]
    for point in points:
        coordinate = symmetry_coordinate(point)
        point_list.append(coordinate)
    return point_list

def symmetry_coordinate(point):
    point = point.upper()
    coordinate = point_coordinate[point]
    return coordinate

# def band_input(points,num_points):
#     coordinates = band_points(points)
#     parameter = []
#     for j,i in enumerate(coordinates):
#         input_line = {'x':str(i[0]),'y':str(i[1]),'z':str(i[2]),'number':str(num_points),'label':f" ! {points[j].upper()}"}
#         parameter.append(input_line)
#     return parameter


def band_input(path,points,num_points):
    parameter = []
    for k,i in enumerate(path):
        for j in points:
            if i.upper() == j[0].upper():
                if k == len(path)-1:
                    input_line = {'x':str(j[1]),'y':str(j[2]),'z':str(j[3]),'number':str(1),'label':f" ! {j[0].upper()}"}
                else:
                    input_line = {'x':str(j[1]),'y':str(j[2]),'z':str(j[3]),'number':str(num_points),'label':f" ! {j[0].upper()}"}
                parameter.append(input_line)
    return parameter



from . import scaffold
from . import reads
from . import utils
from . import check
import os
import sys


def create_folders(project_id,job_id):
    #Create directory for input files
    try:
        os.makedirs(f'./Projects/')
    except:
        pass
    try:
        os.makedirs(f'./Projects/{project_id}')
        os.makedirs(f'./Projects/{project_id}/{job_id}')
    except:
        try:
            os.makedirs(f'./Projects/{project_id}/{job_id}')
        except:
            pass

def pw_checks(self):
    project_id = self.project_id
    job_id = self.job_id
    config=self.config['pw']
    calculation=self.calculation
    check.project_id(project_id)
    check.job_id(job_id)
    check.config(config)
    config['control']['calculation'] = self.calculation
    # Set outfolders
    create_folders(project_id=project_id,job_id=job_id)
    config["control"]["outdir"] = f"./Projects/{project_id}/{job_id}/"
    
    cell, atoms = config['cell_parameters'], config['atomic_positions']


    config["control"]['prefix'] = job_id
    config["system"]['nat'] = len(atoms)
    #check types of atoms
    try:
        config["system"]['ntyp']
    except:
        config["system"]['ntyp'] = utils.atom_type(atoms)
    
    #check ibrav
    try:
        config["system"]['ibrav']
    except:
        config["system"]['ibrav'] = 0
    config["control"]['calculation'] = calculation

    

def input(self):
    if self.package=='pw':
        pw_checks(self)
    elif self.package=='ph':
        ph_checks(self)
    elif self.package=='q2r':
        q2r_checks(self)
    elif self.package=='matdyn':
        matdyn_checks(self)
    elif self.package=='projwfc':
        projwfc_checks(self)
    elif self.package=='bands':
        bands_checks(self)
    else:
        generic_check(self)
    scaffold.constructor(self)

def projwfc_checks(self):
    if self.calculation=='pdos':
        self.config[self.package][self.package][f"filpdos"]  = f'./Projects/{self.project_id}/{self.job_id}/{self.package}.dat'
    elif self.calculation=='kdos':
        self.config[self.package][self.package]["filpdos"]  = f'./Projects/{self.project_id}/{self.job_id}/dos.k'
        self.config[self.package][self.package]["kresolveddos"]  = 'true'
        self.config[self.package][self.package]["degauss"]  =  self.config['pw']['system']['degauss']
    self.config[self.package][self.package]["outdir"] = f"./Projects/{self.project_id}/{self.job_id}/"
    self.config[self.package][self.package]["prefix"] = self.job_id

def bands_checks(self):
    self.config[self.package][self.package]["outdir"] = f"./Projects/{self.project_id}/{self.job_id}/"
    self.config[self.package][self.package]["prefix"] = self.job_id
    self.config[self.package][self.package][f"filband"]  = f'./Projects/{self.project_id}/{self.job_id}/{self.package}.dat'


def generic_check(self):
    self.config[self.package][self.package]["outdir"] = f"./Projects/{self.project_id}/{self.job_id}/"
    self.config[self.package][self.package]["prefix"] = self.job_id
    self.config[self.package][self.package][f"fil{self.package}"]  = f'./Projects/{self.project_id}/{self.job_id}/{self.package}.dat'


def ph_checks(self):
    self.config['ph']["inputph"]["outdir"] = f"./Projects/{self.project_id}/{self.job_id}/"
    self.config['ph']["inputph"]['prefix'] = self.job_id
    self.config['ph']["inputph"]["fildyn"]= f'./Projects/{self.project_id}/{self.job_id}/{self.job_id}.dyn'
def q2r_checks(self):
        self.config['q2r']['input']['fildyn'] = f'./Projects/{self.project_id}/{self.job_id}/{self.job_id}.dyn'
        self.config['q2r']['input']['flfrc']  = f'./Projects/{self.project_id}/{self.job_id}/{self.job_id}.fc'
def matdyn_checks(self):
        self.config['matdyn']['input']['fildyn'] = f'./Projects/{self.project_id}/{self.job_id}/{self.job_id}.dyn'
        self.config['matdyn']['input']['flfrc']  = f'./Projects/{self.project_id}/{self.job_id}/{self.job_id}.fc'
        self.config['matdyn']['input']['flfrq']  = f'./Projects/{self.project_id}/{self.job_id}/{self.job_id}.freq'
        self.config['matdyn']['input']['flvec']  = f'./Projects/{self.project_id}/{self.job_id}/matdyn.modes'


def runner(project_name,iteration,file_names,calculation,qe_path,ncore):
    pre = f'''QE={qe_path}
SLURM_NTASKS={ncore}
project_name={project_name}
    '''
    # print(pre)
    post =f'''
echo Starting {iteration} calculation
for file_name in {" ".join(file_names)}
do
echo $file_name is started
mpirun -n $SLURM_NTASKS $QE/pw.x -inp ./$project_name/$file_name/{calculation}.in > ./$project_name/$file_name/{calculation}.out
echo $file_name is done
done
echo All {iteration} calculations are completed
    '''
    # print(post)
    with open(f"./{iteration}.sh", 'w') as file:
        file.write(pre)
        file.write(post)
    return


from dotenv import dotenv_values

config = dotenv_values(".env")

QE_PATH=config['QE_PATH']
COMPILER=config['COMPILER']
SLURM_JOB_NUM_NODES=config['SLURM_JOB_NUM_NODES']             
SLURM_NTASKS=config['SLURM_NTASKS']
SLURM_PARTITION=config['SLURM_PARTITION']
SLURM_JOB_NAME=config['SLURM_JOB_NAME']
WORK_DIR=config['WORK_DIR']
PYTHON_ENV=config['PYTHON_ENV']


def generate(file_name="run.sh",num_core=SLURM_NTASKS,num_node=SLURM_JOB_NUM_NODES,job_name=SLURM_JOB_NAME,partition=SLURM_PARTITION,qe_path=QE_PATH,compiler_path=COMPILER,PYTHON_ENV=PYTHON_ENV):
    text=f"""{sbatch(num_core,num_node,job_name,partition)}
{qe(qe_path)}
{compiler(compiler_path)}
{python(PYTHON_ENV)}
cd {WORK_DIR}
python3 job.py
"""
    with open(file_name, 'w') as file:
        file.write(text)
    return text


def sbatch(num_core,num_node,job_name,partition):
    text = f"""#!/bin/bash 
#SBATCH --partition {partition}
#SBATCH --nodes     {num_node}
#SBATCH --ntasks    {num_core}
#SBATCH --job-name  {job_name}
#SBATCH --output    {job_name}.out
"""
    return text

def qe(path):
    text = f"""
export PATH="{path}/bin/:$PATH"
"""
    return text
def python(path):
    text=f"""
{path}
""" 
    return text

def compiler(path):
    text = f"""
{path}
"""
    return text
